@@ -103,7 +103,7 @@ sub STORESIZE {
 		(CORE::substr( $_[0]->[REF], $_[1] * $_[0]->[SIZE] ) = '' )
 }
 sub DELETE    { warn "This array does not implement delete functionality" }
-sub PUSH      { $_[0]->[REF] .= pack( $_[0]->[PACK] . '*', @_[1..$#_] ) if $#_ }
+#sub PUSH      { $_[0]->[REF] .= pack( $_[0]->[PACK] . '*', @_[1..$#_] ) if $#_ }
 sub POP       { length($_[0]->[REF]) ? unpack( $_[0]->[PACK], CORE::substr( $_[0]->[REF], -$_[0]->[SIZE], $_[0]->[SIZE], '' )) : undef }
 sub SHIFT     { length($_[0]->[REF]) ? unpack( $_[0]->[PACK], CORE::substr( $_[0]->[REF], 0, $_[0]->[SIZE], '' )) : undef }
 sub UNSHIFT   { CORE::substr( $_[0]->[REF], 0, 0, pack( $_[0]->[PACK] . '*', @_[1..$#_] )) if $#_ }
@@ -388,7 +388,7 @@ sub set
 	$o-> $w( \%pr);
 }
 
-sub colored { $_[0]->{OWNER}->is_font_colored }
+sub has_color { $_[0]->{OWNER}->font_has_color }
 
 for ( qw( size name width height direction style pitch encoding vector)) {
 	eval <<GENPROC;
@@ -422,7 +422,7 @@ sub _debug
 {
 	my $f = shift;
 	my $noprint = shift;
-	my $r = sprintf "%d.%s [%dx%d]",
+	my $r = sprintf "%g.%s [%dx%d]",
 		@{$f}{qw(size name width height)};
 	$r .= " thin"   if $f->{style} & fs::Thin;
 	$r .= " bold"   if $f->{style} & fs::Bold;
@@ -467,7 +467,7 @@ eval <<PROP for qw( A B C D X Y );
 	}
 PROP
 
-eval <<METH for qw( identity translate scale shear rotate transform inverse_transform);
+eval <<METH for qw( identity translate scale shear rotate transform inverse_transform multiply);
 	sub $_
 	{
 		my \$self = shift;
@@ -822,7 +822,10 @@ sub graphic_context
 	my $self = shift;
 	my $cb   = pop;
 	return unless $self->graphic_context_push;
-	$self->set(@_);
+	my %p = @_;
+	$self->add_region(delete $p{region}) if exists $p{region};
+	$self->matrix->multiply( delete $p{matrix} ) if exists $p{matrix};
+	$self->set(%p);
 	my $ok = $cb->();
 	return unless $self->graphic_context_pop;
 	return $ok;
@@ -1311,6 +1314,8 @@ sub notification_types { return \%RNT; }
 	hiliteColor       => cl::HiliteText,
 	hint              => '',
 	hintVisible       => 0,
+	hintBackColor     => 0xffff80,
+	hintColor         => cl::Black,
 	layered           => 0,
 	light3DColor      => cl::Light3DColor,
 	left              => 100,
@@ -1318,6 +1323,8 @@ sub notification_types { return \%RNT; }
 	ownerBackColor    => 0,
 	ownerFont         => 1,
 	ownerHint         => 1,
+	ownerHintColor    => 1,
+	ownerHintBackColor=> 1,
 	ownerShowHint     => 1,
 	ownerSkin         => 1,
 	ownerPalette      => 1,
@@ -2360,7 +2367,21 @@ sub profile_default
 	return $def;
 }
 
-sub max_extents { map { $_ / 2 } $::application->size }
+sub max_extents
+{
+	shift;
+	my @pos = @_ || $::application-> pointerPos;
+	my @sz = $::application->size;
+	for my $r ( @{$::application->get_monitor_rects} ) {
+		next unless
+			$r->[0] <= $pos[0] && $r->[2] > $pos[0] &&
+			$r->[1] <= $pos[1] && $r->[3] > $pos[1]
+		;
+		@sz = ( $r->[2] - $r->[0], $r->[3] - $r->[1] );
+		last;
+	}
+	return map { $_ / 2 } @sz;
+}
 
 sub on_change
 {
@@ -2473,14 +2494,15 @@ sub profile_default
 		ownerColor     => 0,
 		ownerBackColor => 0,
 		ownerFont      => 0,
+		ownerHint      => 0,
+		ownerHintColor => 0,
+		ownerHintBackColor => 0,
 		ownerShowHint  => 0,
 		ownerSkin      => 0,
 		ownerPalette   => 0,
 		showHint       => 1,
 		skin           => 'flat',
 		hintClass      => 'Prima::HintWidget',
-		hintColor      => cl::Black,
-		hintBackColor  => 0xffff80,
 		hintPause      => 800,
 		hintFont       => Prima::Widget::get_default_font,
 		modalHorizon   => 1,
@@ -2502,13 +2524,10 @@ sub profile_check_in
 	$p->{textDirection} //= $self->lang_is_rtl($p->{language} // $default->{language});
 	$GUI_EXCEPTION = delete $p->{guiException} if exists $p->{guiException};
 	$self-> SUPER::profile_check_in( $p, $default);
-	delete $p-> { printerModule};
-	delete $p-> { owner};
-	delete $p-> { ownerColor};
-	delete $p-> { ownerBackColor};
-	delete $p-> { ownerFont};
-	delete $p-> { ownerShowHint};
-	delete $p-> { ownerPalette};
+	delete @{$p}{qw(
+		printerModule owner ownerColor ownerBackColor ownerFont ownerHint
+		ownerHintColor ownerHintBackColor ownerShowHint ownerPalette)
+	};
 }
 
 sub add_startup_notification
