@@ -25,12 +25,12 @@ use constant Y => 5;
 sub new
 {
 	my ( $class, $canvas, %opt ) = @_;
+	Carp::cluck("Option 'antialias' is deprecated, use 'subpixel' instead") if exists $opt{antialias};
 	return bless {
 		canvas          => $canvas,
 		commands        => [],
 		precision       => undef,
-		subpixel        => $opt{antialias} // (($canvas && ref($canvas)) ? $canvas->antialias : 0),
-		antialias       => 0,
+		subpixel        => $opt{subpixel} // (($canvas && ref($canvas)) ? $canvas->antialias : 0),
 		%opt
 	}, $class;
 }
@@ -76,7 +76,8 @@ sub moveto           { shift->cmd('moveto', shift, shift, 0) }
 sub rmoveto          { shift->cmd('moveto', shift, shift, 1) }
 sub restore          { shift->cmd('restore') } # no checks for underflow here, to allow append paths
 sub precision        { shift->cmd(set => precision => shift) }
-sub antialias        { $#_ ? $_[0]->{antialias} = $_[1] : $_[0]->{antialias} }
+sub subpixel         { $#_ ? $_[0]->{subpixel} = $_[1] : $_[0]->{subpixel} }
+sub antialias        { Carp::cluck("Property 'antialias' is deprecated, use 'subpixel' instead") }
 
 sub reset
 {
@@ -182,37 +183,85 @@ sub rspline
 	$self-> rcmd( spline => $p, \%opt );
 }
 
-sub glyph
+sub _outline2path
 {
-	my ($self, $ix, %opt) = @_;
-	return unless $self->{canvas};
-	my $outline = $self->{canvas}->render_glyph( $ix, %opt );
-	return unless $outline;
-	my $size = scalar(@$outline);
+	my ( $self, $fill, $outline ) = @_;
+
 	my @p;
-	my $fill = delete $opt{fill};
+	my @anchor;
+	my $do_anchor;
+	my $size = scalar(@$outline);
 	for ( my $i = 0; $i < $size; ) {
 		my $cmd = $outline->[$i++];
 		my $pts = $outline->[$i++] * 2;
 		my @pts = map { $outline->[$i++] / 64.0 } 0 .. $pts - 1;
 		if ( $cmd == ggo::Move ) {
 			$self->close unless $fill;
-			$self->moveto(@pts);
+			$self->moveto(@anchor = @pts);
+			$do_anchor = 0;
 		} elsif ( $cmd == ggo::Line ) {
 			$self->line([ @p, @pts ]);
+			$do_anchor = 1;
 		} elsif ( $cmd == ggo::Conic ) {
 			$self->spline([ @p, @pts ]);
+			$do_anchor = 1;
 		} elsif ( $cmd == ggo::Cubic ) {
 			$self->spline([ @p, @pts ], degree => 3 );
+			$do_anchor = 1;
+		} else {
+			next;
 		}
 		@p = @pts[-2,-1];
 	}
+	$self->line(@anchor) if $do_anchor && @anchor;
 	$self->close;
 }
 
+sub glyph
+{
+	my ($self, $ix, %opt) = @_;
+	return unless $self->{canvas};
+	my $outline = $self->{canvas}->render_glyph( $ix, %opt );
+	return unless $outline;
+
+	my $fill = delete $opt{fill};
+
+	if ( !$opt{color} ) {
+		$self->_outline2path($fill, $outline);
+		return $self;
+	}
+
+	my @ret   = ( $self->{canvas}->new_path );
+	my $size  = scalar(@$outline);
+	my ($i,$start);
+	for ( $i = $start = 0; $i < $size; ) {
+		my $cmd = $outline->[$i++];
+		my $pts = $outline->[$i++] * 2;
+		if ( $cmd != ggo::SetColor ) {
+			$i += $pts;
+			next;
+		}
+		if ( $start < $i - 2 ) {
+			my @sub_outline = @$outline[ $start .. $i - 3 ];
+			$ret[-1]->_outline2path( $fill, \@sub_outline);
+		}
+		push @ret, $outline->[$i], $self->{canvas}->new_path
+			unless @ret > 1 && $outline->[$i] == $ret[-2];
+		$i += $pts;
+		$start = $i;
+	}
+	if ( $start != $i ) {
+		my @sub_outline = @$outline[ $start .. $#$outline ];
+		$ret[-1]->_outline2path( $fill, \@sub_outline);
+	}
+
+	return @ret;
+}
+
 sub text
 {
 	my ($self, $text, %opt) = @_;
+	return if $opt{color};
 	return unless my $c = $self->{canvas};
 	my $state = $c->get_paint_state;
 	unless ($state) {
@@ -249,7 +298,11 @@ sub arc
 	@_ > 5 or Carp::croak('bad parameters to arc');
 	my ( $cx, $cy, $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
-	$dx--, $dy-- unless $self->{subpixel};
+	unless ($self->{subpixel}) {
+		$dx--;
+		$dy--;
+		return $self if $dx <= 0 || $dy <= 0;
+	}
 	if ( $tilt // 0.0 ) {
 		return $self-> save->
 			scale( $dx / 2, $dy / 2)->
@@ -271,7 +324,11 @@ sub rarc
 	@_ > 3 or Carp::croak('bad parameters to arcto');
 	my ( $dx, $dy, $from, $to, $tilt) = @_;
 	return $self if $from == $to;
-	$dx--, $dy-- unless $self->{subpixel};
+	unless ($self->{subpixel}) {
+		$dx--;
+		$dy--;
+		return $self if $dx <= 0 || $dy <= 0;
+	}
 	$self->save;
 	$self->scale( $dx / 2, $dy / 2);
 	$self->rotate( $tilt // 0.0);
@@ -285,7 +342,11 @@ sub ellipse
 	@_ > 2 or Carp::croak('bad parameters to ellipse');
 	my ( $cx, $cy, $dx, $dy, $tilt) = @_;
 	$dy //= $dx;
-	$dx--, $dy-- unless $self->{subpixel};
+	unless ($self->{subpixel}) {
+		$dx--;
+		$dy--;
+		return $self if $dx <= 0 || $dy <= 0;
+	}
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		rotate( $tilt // 0.0)->
@@ -298,7 +359,11 @@ sub chord
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to chord');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
-	$dx--, $dy-- unless $self->{subpixel};
+	unless ($self->{subpixel}) {
+		$dx--;
+		$dy--;
+		return $self if $dx <= 0 || $dy <= 0;
+	}
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		circular_arc( $start, $end )->
@@ -312,9 +377,10 @@ sub lines
 	my $p = $#_ ? [@_] : $_[0];
 	@$p % 4 and Carp::croak('bad parameters to lines');
 	for ( my $i = 0; $i < @$p; $i += 4 ) {
-		$self->moveto(@$p[ $i .. $i + 1 ]);
+		$self->open;
 		$self->cmd( line   => [ @$p[ $i .. $i + 3 ] ] );
 	}
+	$self->open;
 }
 
 sub rectangle
@@ -330,7 +396,11 @@ sub sector
 	my $self = shift;
 	@_ == 6 or Carp::croak('bad parameters to sector');
 	my ( $cx, $cy, $dx, $dy, $start, $end) = @_;
-	$dx--, $dy-- unless $self->{subpixel};
+	unless ($self->{subpixel}) {
+		$dx--;
+		$dy--;
+		return $self if $dx <= 0 || $dy <= 0;
+	}
 	$self-> save->
 		matrix( $dx / 2, 0, 0, $dy / 2, $cx, $cy )->
 		line(0,0)->
@@ -342,13 +412,15 @@ sub sector
 sub round_rect
 {
 	my ( $self, $x, $y, $x1, $y1, $maxd) = @_;
+	return $self->rectangle($x, $y, $x1, $y1) if $maxd <= 0;
+
 	( $x1, $x) = ( $x, $x1) if $x > $x1;
 	( $y1, $y) = ( $y, $y1) if $y > $y1;
 	my ( $dx, $dy) = ( $x1 - $x, $y1 - $y);
 	$dx = $maxd if $dx > $maxd;
 	$dy = $maxd if $dy > $maxd;
 	my $d = ( $dx < $dy ) ? $dx : $dy;
-	my $r = $self->{subpixel} ? $d/2 : ((int($d/4) * 2) + 1);
+	my $r = $d/2;
 #  plots roundrect:
 # A'        B'
 #  /------\
@@ -378,40 +450,77 @@ sub round_rect
 
 sub new_array { shift->{subpixel} ? Prima::array->new_double : Prima::array->new_int }
 
+sub new_contour
+{
+	my $self = shift;
+	my $p = $self->{points};
+	while (@$p && @{$p->[-1]} < 4) {
+		$self->{hint_map}->[ $#$p ] = undef
+			if exists $self->{hint_map};
+		pop @$p;
+	}
+	push @$p, $self->new_array;
+}
+
 sub points
 {
 	my ($self, %opt) = @_;
+
+	my ($with_hints, $hints);
+	if ( $opt{hints}) {
+		delete $self->{points};
+		$with_hints = 1;
+	}
+
 	unless ( $self->{points} ) {
 		local $self->{stack} = [];
 		local $self->{curr}  = {
 			matrix => [ identity ],
 			( map { $_, $self->{$_} } qw(precision) )
 		};
-		$self->{points} = [[ $self->new_array ]];
+		my $p = $self->{points} = [ $self->new_array ];
+
+		my ( $lj_signal, $lj_override ) = (0,0);
 		my $c = $self->{commands};
+		$hints = $self->{hint_map} = [] if $with_hints;
+
 		for ( my $i = 0; $i < @$c; ) {
+
 			my ($cmd,$len) = @$c[$i,$i+1];
+
+			my ($np, $got_hint);
+			if ( $with_hints ) {
+				if ( $cmd eq 'spline' || $cmd eq 'arc') {
+					$lj_signal = 1;
+				} elsif ( $cmd eq 'line') {
+					$lj_signal = 0;
+				}
+				if ( $lj_signal != $lj_override ) {
+					push @{ $hints->[$#$p] }, scalar(@{$p->[-1]}) / 2;
+					$lj_override = $lj_signal;
+					$got_hint = 1;
+				}
+				$np = @$p;
+			}
+
 			$self-> can("_$cmd")-> ( $self, @$c[$i+2..$i+$len+1] );
 			$i += $len + 2;
+
+			$lj_signal = $lj_override = $got_hint = 0 if $with_hints &&
+				( $np != @$p || ($got_hint && !defined( $hints->[$#$p] )));
 		}
-		for my $ppp ( @{$self->{points}}) {
-			@$ppp = grep { @$_ > 2 } @$ppp;
-			Prima::array::deduplicate($_,2,4) for @$ppp;
-		}
+		$self-> new_contour;
+		pop @$p;
+
 		$self->{last_matrix} = $self->{curr}->{matrix};
 	}
 
-	if ( $opt{fill} ) {
-		my @ret;
-		for my $ppp ( @{ $self->points } ) {
-			my $arr = $self->new_array;
-			Prima::array::append( $arr, $_ ) for @$ppp;
-			push @ret, $arr if @$arr > 2;
-		}
-		return @ret;
+	if ( $opt{hints}) {
+		delete $self->{hint_map};
+		return $self->{points}, $hints;
 	}
 
-	return $self->{points};
+	return wantarray ? @{$self->{points}} : $self->{points};
 }
 
 sub last_matrix
@@ -424,9 +533,7 @@ sub last_matrix
 sub last_point
 {
 	for ( reverse @{ shift->{points} }) {
-		for ( reverse @$_ ) {
-			return $$_[-2], $$_[-1] if @$_;
-		}
+		return $$_[-2], $$_[-1] if @$_;
 	}
 	return 0,0;
 }
@@ -494,44 +601,45 @@ sub  _moveto
 	my ( $self, $mx, $my, $rel) = @_;
 	($mx, $my) = $self->matrix_apply($mx, $my);
 	my ($lx, $ly) = $rel ? $self->last_point : (0,0);
-	my $arr = $self->new_array;
-	push @$arr, $self->{subpixel} ? ($lx + $mx, $ly + $my) : Prima::Utils::nearest_i($lx + $mx, $ly + $my);
-	push @{$self->{points}->[-1]}, $arr;
+	push @{$self->{points}->[-1]},
+		$self->{subpixel} ?
+			($lx + $mx, $ly + $my) :
+			Prima::Utils::nearest_i($lx + $mx, $ly + $my);
 }
 
-sub _open { push @{$_[0]->{points}}, [$_[0]->new_array] }
+sub _open { shift-> new_contour }
 
 sub _close
 {
 	my $self = shift;
 	my $p = $self->{points};
 	return unless @$p;
-	my $l = $p->[-1]->[-1];
+	my $l = $p->[-1];
 	push @$l, $$l[0], $$l[1] if @$l && ($$l[0] != $$l[-2] || $$l[1] != $$l[-1]);
-	push @$p, [$self->new_array];
+	$self-> new_contour;
 }
 
 sub _line
 {
 	my ( $self, $line ) = @_;
-	Prima::array::append( $self->{points}->[-1]->[-1],
-		Prima::Drawable->render_polyline( $line,
-			matrix  => $self->{curr}->{matrix},
-			integer => !$self->{subpixel},
-		)
+	my $ppp = Prima::Drawable->render_polyline( $line,
+		matrix  => $self->{curr}->{matrix},
+		integer => !$self->{subpixel},
 	);
+	Prima::array::deduplicate($ppp,2,4);
+	Prima::array::append( $self->{points}->[-1], $ppp);
 }
 
 sub _spline
 {
 	my ( $self, $points, $options ) = @_;
-	Prima::array::append( $self->{points}->[-1]->[-1],
-		Prima::Drawable->render_spline(
-			$self-> matrix_apply( $points ),
-			%$options,
-			integer => !$self->{subpixel},
-		)
+	my $ppp = Prima::Drawable->render_spline(
+		$self-> matrix_apply( $points ),
+		%$options,
+		integer => !$self->{subpixel},
 	);
+	Prima::array::deduplicate($ppp,2,4);
+	Prima::array::append( $self->{points}->[-1], $ppp);
 }
 
 # Reference:
@@ -607,7 +715,7 @@ sub _arc
 
 	my %xopt;
 	$xopt{precision} = $self->{curr}->{precision} // 24;
-	$xopt{precision} = $scaling if $xopt{precision} // 24 > $scaling;
+	$xopt{precision} = $scaling if $xopt{precision} > $scaling;
 	$xopt{integer}   = !$self->{subpixel};
 	$xopt{precision} *= 2 if
 		$self->{subpixel} &&
@@ -624,28 +732,17 @@ sub _arc
 		} elsif ($xopt{precision} >= 2) {
 			$poly = Prima::Drawable->render_spline( $poly, @options, %xopt);
 		}
-		Prima::array::append( $self->{points}->[-1]->[-1], $poly);
+		Prima::array::deduplicate($poly,2,4);
+		Prima::array::append( $self->{points}->[-1], $poly);
 	}
 }
 
-sub acquire
-{
-	my $c = $_[0]->{canvas};
-	$_[0]->{subpixel} = $c->antialias || $c->alpha < 255;
-}
-
 sub stroke
 {
 	return 0 unless $_[0]->{canvas};
-	$_[0]->acquire;
-	my $emulated_aa = $_[0]->{antialias} && !$_[0]->{canvas}->antialias;
-	for ( map { @$_ } @{ $_[0]->points }) {
+	for ( $_[0]->points ) {
 		next if 4 > @$_;
-		if ( $emulated_aa ) {
-			return 0 unless $_[0]->{canvas}->new_aa_surface->polyline($_);
-		} else {
-			return 0 unless $_[0]->{canvas}->polyline($_);
-		}
+		return 0 unless $_[0]->{canvas}->polyline($_);
 	}
 	return 1;
 }
@@ -654,26 +751,73 @@ sub fill
 {
 	my ( $self, $fillMode ) = @_;
 	return 0 unless my $c = $self->{canvas};
-	$self->acquire;
-	my @p = $self->points(fill => 1);
 	my $ok = 1;
 	my $save;
 	if ( defined $fillMode ) {
 		$save = $c->fillMode;
 		$c->fillMode($fillMode);
 	}
-	for ( @p ) {
+	for ( $self->points ) {
 		next if 4 > @$_;
-		if ( $self->{antialias} && !$_[0]->{canvas}->antialias) {
-			last unless $ok &= $c->new_aa_surface->fillpoly($_);
-		} else {
-			last unless $ok &= $c->fillpoly($_);
-		}
+		last unless $ok &= $c->fillpoly($_);
 	}
 	$c->fillMode($save) if defined $save;
 	return $ok;
 }
 
+sub fill_gradient
+{
+	my ( $self, %opt ) = @_;
+	return unless my $c = $self->{canvas};
+
+	my ($method, @xopt);
+	my $filler = delete $opt{type} // 'horizontal';
+	if ( $filler eq 'horizontal') {
+		$method = 'bar';
+		push @xopt, 0;
+	} elsif ( $filler eq 'vertical') {
+		$method = 'bar';
+		push @xopt, 1;
+	} elsif ( $filler eq 'circular') {
+		$method = 'ellipse';
+	} elsif ( $filler eq 'radial') {
+		$method = 'sector';
+	} else {
+		Carp::carp("unknown filler type: $filler");
+		return;
+	}
+
+	my $fm       = delete $opt{fill} // (fm::Winding | fm::Overlay);
+	my $gradient = $c->new_gradient(%opt);
+
+	$c->graphic_context( sub {
+		my $rgn = Prima::Region->new( polygons => scalar($self->points), fillMode => $fm, matrix => $c->get_matrix);
+		$c->add_region( $rgn );
+		$c->matrix->identity;
+
+		$c->antialias(0);
+		if ( $method eq 'bar') {
+			$gradient->bar( $rgn->rect, @xopt );
+		} elsif ( $method eq 'ellipse') {
+			my @box = $rgn->box;
+			my $d = int(sqrt( $box[2]*$box[2] + $box[3]*$box[3] ) + .5);
+			$gradient->ellipse(
+				$box[0] + $box[2] / 2,
+				$box[1] + $box[3] / 2,
+				$d, $d
+			);
+		} elsif ( $method eq 'sector') {
+			my @box = $rgn->box;
+			my $d = int(sqrt( $box[2]*$box[2] + $box[3]*$box[3] ) + .5);
+			$gradient->sector(
+				$box[0] + $box[2] / 2,
+				$box[1] + $box[3] / 2,
+				$d, $d, 0, 360
+			);
+		}
+	} );
+}
+
 sub fill_stroke
 {
 	my ( $self, $fillMode ) = @_;
@@ -872,7 +1016,7 @@ sub contours
 {
 	my $self = shift;
 	my @ret;
-	for my $pp ( map { @$_ } @{ $self->points } ) {
+	for my $pp ( $self->points ) {
 		my @contour;
 		next if @$pp < 2;
 		my $closed = $pp->[0] == $pp->[-2] && $pp->[1] == $pp->[-1];
@@ -1035,10 +1179,12 @@ sub poly2patterns
 
 # Adapted from wine/dlls/gdi32/path.c:WidenPath()
 # (c) Martin Boehme, Huw D M Davies, Dmitry Timoshkov, Alexandre Julliard
+#
+# I keep this for easy debugging and reference, however this implementation
+# doesn't include the line join hinting that is present in render_polyline()
 sub widen_old
 {
 	my ( $self, %opt ) = @_;
-	$self->acquire;
 
 	my $dst = ref($self)->new( undef,
 		%$self,
@@ -1057,7 +1203,7 @@ sub widen_old
 	my $pp;
 	{
 		local $self->{subpixel} = 1;
-		$pp = [ map { @$_ } @{$self->points} ];
+		$pp = $self->points;
 	}
 	return $dst if $lp eq lp::Null;
 	$pp = poly2patterns($pp, $lp, $lw, !$self->{subpixel}) if $lp ne lp::Solid;
@@ -1083,6 +1229,10 @@ sub widen_old
 
 	my @dst;
 	my $lw2 = $lw / 2;
+
+	# classic widening algorithm doesn't do that (I believe) because there's no fm::Overlay - but Prima has it
+	$lw2 -= 0.5 unless $self->{subpixel};
+
 	for ( @$pp ) {
 		next unless @$_;
 
@@ -1240,7 +1390,6 @@ sub widen_old
 sub widen_new
 {
 	my ( $self, %opt ) = @_;
-	$self->acquire;
 
 	my $dst = ref($self)->new( undef,
 		%$self,
@@ -1248,17 +1397,20 @@ sub widen_new
 		commands => [],
 	);
 
-	my @pp;
+	my ($pp, $hints);
 	{
 		local $self->{subpixel} = 1;
-		@pp = map { @$_ } @{$self->points};
+		($pp, $hints) = $self-> points( hints => 1);
 	}
 
-	for my $p ( @pp ) {
-		next unless @$p;
+	for (my $j = 0; $j < @$pp; $j++ ) {
+		my $p = $pp->[$j];
+		my $h = $hints->[$j];
+
 		my $cmds = $self->{canvas}->render_polyline( $p, %opt,
 			path    => 1,
-			integer => $self->{subpixel} ? 0 : 1,
+			integer => ($self->{subpixel} ? 0 : 1),
+			line_join_hints => $h,
 		);
 
 		for ( my $i = 0; $i < @$cmds;) {
@@ -1290,7 +1442,7 @@ sub widen_new
 sub extents
 {
 	my $self = shift;
-	my @pp = map { @$_ } @{ $self->points };
+	my @pp = $self->points;
 	return unless @pp;
 	my ( $x1, $y1, $x2, $y2 ) = @{$pp[0]}[0,1,0,1];
 	for my $p ( @pp ) {
@@ -1317,19 +1469,15 @@ sub clip
 	$p->clear;
 	$p->set(%opt) if scalar keys %opt;
 	$p->translate($tx, $ty);
-	$p->fillpoly($_) for $self->points(fill => 1);
+	$p->fillpoly($_) for $self->points;
 	return $p->image;
 }
 
 sub region
 {
-	my ($self, $mode, $rgnop) = @_;
-	my $reg;
+	my ($self, $mode) = @_;
 	$mode //= fm::Winding | fm::Overlay;
-	$rgnop //= rgnop::Union;
-	$reg ? $reg->combine($_, $rgnop) : ($reg = $_)
-		for map { Prima::Region->new( polygon => $_, fillMode => $mode) } $self->points(fill => 1);
-	return $reg // Prima::Region->new;
+	return Prima::Region->new( polygons => scalar($self->points), fillMode => $mode);
 }
 
 sub _debug_commands
@@ -1349,19 +1497,21 @@ sub _debug_commands
 
 __END__
 
+=pod
+
 =head1 NAME
 
 Prima::Drawable::Path - stroke and fill complex paths
 
 =head1 DESCRIPTION
 
-The module augments the C<Prima::Drawable> drawing and plotting functionality by
-implementing paths that allow arbitrary combination of polylines, splines, and arcs,
+The module augments the C<Prima::Drawable>'s drawing and plotting functionality by
+implementing paths that allow arbitrary combinations of polylines, splines, and arcs,
 to be used for drawing or clipping shapes.
 
 =head1 SYNOPSIS
 
-	# draws elliptic spiral
+	# draws an elliptic spiral
 	my ( $d1, $dx ) = ( 0.8, 0.05 );
 	$canvas-> new_path->
 		rotate(45)->
@@ -1391,47 +1541,57 @@ the previous primitive (or (0,0) if there's none).
 
 =item arc CENTER_X, CENTER_Y, DIAMETER_X, DIAMETER_Y, ANGLE_START, ANGLE_END, TILT = 0
 
-Adds elliptic arc to path centered around (CENTER_X,CENTER_Y).
+Adds an elliptic arc to the path. The arc is centered around the (CENTER_X,CENTER_Y) point.
 
-Important: if the intention is an immediate rendering, especially with 1-pixel
+Important: if the intention is an immediate rendering, especially with a 1-pixel
 line width, consider decreasing diameters by 1. This is because all arc
-calculations are made with floating point, where diameter is also given not in
-pixels but in geometrical coordinates, to allow for matrix transformations.
-Before rendering is performed, arcs are tranformed into spline vertices and
-then transformation matrix is applied, and by that time the notion of an arc
-diameter is lost to be successfully converted into pixel size minus one.
+calculations are made with the floating point precision, where the diameter is also
+given not in pixels but in geometrical coordinates, to allow for matrix
+transformations.  Before rendering is performed, arcs are transformed into
+spline vertices and then the transformation matrix is applied, and by that time the
+notion of an arc diameter is lost to be successfully converted into pixel size
+minus one.
+
+Read more about this in L<Prima::Drawable/Antialiasing and alpha> .
 
 =item close, open
 
-Closes the current shape and opens a new one
-close() is same as open() but makes sure the shape's first point is equal to its last point.
+Closes the current shape and opens a new one.  close() is the same as open() but
+makes sure the shape's first point is equal to its last point.
 
 =item circular_arc ANGLE_START, ANGLE_END
 
-Adds circular arc to the path. Note that adding transformations will effectively
-make it into elliptic arc, which is used internally by C<arc> and C<rarc>.
+Adds a circular arc to the path. Note that adding transformations will effectively
+make it into an elliptic arc, which is used internally by C<arc> and C<rarc>.
 
 =item chord CENTER_X, CENTER_Y, DIAMETER_X, DIAMETER_Y, ANGLE_START, ANGLE_END.
 
-Adds chord to the path. Is there only for compatibility with C<Prima::Drawable>.
+Adds a chord to the path. Is there only for compatibility with C<Prima::Drawable>.
 
 =item ellipse CENTER_X, CENTER_Y, DIAMETER_X, DIAMETER_Y = DIAMETER_X, TILT = 0
 
-Adds full ellipse to the path.
+Adds a full ellipse to the path.
 
 =item glyph INDEX, %OPTIONS
 
-Adds glyph outline to the path. C<%OPTIONS> are passed as is to L<Prima::Drawable/renger_glyph>.
-Note that filled glyphs require C<fillMode> without the C<fm::Overlay> bit set and C<fill> option set
-to generate proper shapes with holes.
+Adds a glyph outline to the path. C<%OPTIONS> are passed as is to
+L<Prima::Drawable/renger_glyph>, except the C<fill> option.
+
+Note that filled glyphs require C<fillMode> without the C<fm::Overlay> bit set,
+and also the C<fill> option set to generate proper shapes with holes.
+
+If the C<color> is option added, reports an array of scalars where each is
+either a new path object, or an integer scalar thet sets the 24-bit RGB color
+for the subsequent path(s) to be painted with. If the integer scalar is -1 that
+means to use the current color.
 
 =item line, rline @POINTS
 
-Adds a polyline to path
+Adds a polyline to the path
 
 =item lines [X1, Y1, X2, Y2]..
 
-Adds set of multiple, unconnected lines to the path. Is there only for
+Adds a set of multiple, unconnected lines to the path. Is there only for
 compatibility with C<Prima::Drawable>.
 
 =item moveto, rmoveto X, Y
@@ -1440,34 +1600,45 @@ Stops plotting the current shape and moves the plotting position to X, Y.
 
 =item rarc DIAMETER_X, DIAMETER_Y, ANGLE_START, ANGLE_END, TILT = 0
 
-Adds elliptic arc to path so that the first point of the arc starts on the last
-point of the previous primitive, or (0,0) if there's none.
+Adds an elliptic arc to the path so that the first point of the arc starts on
+the last point of the previous primitive, or (0,0) if there's none.
 
 =item rectangle X1, Y1, X2, Y2
 
-Adds rectangle to the path. Is there only for compatibility with C<Prima::Drawable>.
+Adds a rectangle to the path. Is there only for compatibility with
+C<Prima::Drawable>.
 
 =item round_rect X1, Y1, X2, Y2, MAX_DIAMETER
 
-Adds round rectangle to the path.
+Adds a round rectangle to the path.
 
 =item sector CENTER_X, CENTER_Y, DIAMETER_X, DIAMETER_Y, ANGLE_START, ANGLE_END
 
-Adds sector to the path. Is there only for compatibility with C<Prima::Drawable>.
+Adds a sector to the path. Is there only for compatibility with C<Prima::Drawable>.
 
 =item spline, rspline $POINTS, %OPTIONS.
 
-Adds B-spline to path. See L<Prima::Drawable/spline> for C<%OPTIONS> descriptions.
+Adds a B-spline to the path. See L<Prima::Drawable/spline> for C<%OPTIONS> descriptions.
 
 =item text TEXT, %OPTIONS
 
-Adds C<TEXT> to the path. C<%OPTIONS> are same as in L<Prima::Drawable/render_glyph>, 
-except that C<unicode> is deduced automatically based on whether C<TEXT> has utf8 bit
-on or off; and an extra option C<cache> with a hash can be used to speed up the function
-with subsequent calls. C<baseline> option is same as L<Prima::Drawable/textOutBaseline>.
+Adds C<TEXT> to the path. C<%OPTIONS> are the same as in L<Prima::Drawable/render_glyph>, 
+except that C<unicode> is deduced automatically based on whether the C<TEXT> has utf8 bit
+on or off. An extra option C<cache> with a hash can be used to speed up the function
+with subsequent calls. The C<baseline> option is the same as L<Prima::Drawable/textOutBaseline>.
+
+Note that filled glyphs require C<fillMode> without the C<fm::Overlay> bit set,
+and also the C<fill> option set to generate proper shapes with holes.
 
 =back
 
+=head2 Transformations
+
+Transformation calls change the current path properties (matrix etc) so that
+all subsequent calls would use them until a call to C<restore> is made. The
+C<save> and C<restore> methods implement the stacking mechanism so that local
+transformations can be made.
+
 =head2 Properties
 
 =over
@@ -1476,31 +1647,16 @@ with subsequent calls. C<baseline> option is same as L<Prima::Drawable/textOutBa
 
 Provides access to the attached drawable object
 
-=back
-
-=head2 Transformations
-
-Transformation calls change the current path properties (matrix etc)
-so that all subsequent calls will use them until a call to C<restore>
-is used. C<save> and C<restore> implement a stacking mechanism, so that
-local transformations can be made.
-
-The final transformations calculate coordinates the new and the existing matrices:
-
-  P' = NewMatrix * P
-
-=over
-
 =item matrix A, B, C, D, Tx, Ty
 
-Applies transformation matrix to the path. The matrix, as used by the module,
-is formed as such:
+Applies a transformation matrix to the path. The matrix, as used by the module,
+is formed as follows:
 
   A  B  0
   C  D  0
   Tx Ty 1
 
-and when applied to 2D coordinates, is calculated as
+When applied to 2D coordinates, the transformed coordinates are calculated as
 
   X' = AX + CY + Tx
   Y' = BX + DY + Ty
@@ -1509,12 +1665,6 @@ and when applied to 2D coordinates, is calculated as
 
 Selects current precision for splines and arcs. See L<Prima::Drawable/spline>, C<precision> entry.
 
-=item antialias BOOLEAN
-
-Turns on and off slow but more visually pleasant antialiased drawing mode.
-
-Default: false
-
 =item restore
 
 Pops the stack entry and replaces the current matrix and graphic properties with it.
@@ -1525,7 +1675,7 @@ Adds rotation to the current matrix
 
 =item save
 
-Duplicates the current matrix and graphic properties and pushes them to the stack.
+Saves the current matrix and graphic properties on the stack.
 
 =item shear X, Y = X
 
@@ -1539,68 +1689,87 @@ Adds scaling to the current matrix
 
 Turns on and off slow but more precise floating-point calculation mode
 
-Default: depends on canvas antialiasing mode
+Default: depends on the canvas antialiasing mode
 
 =item translate X, Y = X
 
-Adds offset to the current matrix
+Adds an offset to the current matrix
 
 =back
 
 =head2 Operations
 
-These methods perform actual path rendering, that was delayed until that, and will
-create an array of points that can be used for actual drawing.
+These methods perform path rendering and create an array of points that can be
+used for drawing
 
 =over
 
 =item clip %options
 
-Returns 1-bit image with clipping mask of the path. C<%options> can be used to
-pass C<fillMode> property that affects the result of the filled shape.
+Returns a 1-bit image with the clipping mask created from the path. C<%options> can be used to
+pass the C<fillMode> property that affects the result of the filled shape.
 
 =item contours
 
-Same as L<points> but further reduces lines into a 8-connected set of points,
+Same as L<points> but further reduces lines into a set of 8-connected points,
 suitable to be traced pixel-by-pixel.
 
 =item extents
 
-Returns 2 points that box the path.
+Returns two points that box the path.
 
 =item last_matrix
 
-Return CTM resulted after running all commands
+Returns the current transform matrix (CTM) after running all commands
 
 =item fill fillMode=undef
 
 Paints a filled shape over the path. If C<fillMode> is set, it is used instead of the one
 selected on the canvas.
 
+=item fill_gradient %OPTIONS
+
+Applies the path as a region, then calls a gradient filler on it.
+
+Recognizes the following options:
+
+=over
+
+=item type = horizontal
+
+One of: vertical, horizontal, circular, radial
+
+=item fill = fm::Overlay | fm::Winding
+
+The fillMode, if any
+
+=back
+
+Other options are passed as is to the C<new_gradient> method; see L<Prima::Drawable::Gradient> for their description
+
 =item fill_stroke fillMode=undef
 
-Paints a filled shape over the path with back color. If C<fillMode> is set, it is used instead of the one
-selected on the canvas. Thereafter, draws a polyline over the path.
+Paints a filled shape over the path with the background color. If C<fillMode>
+is set, it is used instead of the one selected on the canvas. Thereafter, draws
+a polyline over the path.
 
-=item flatten PRESCALE 
+=item flatten PRESCALE
 
 Returns new objects where arcs are flattened into lines. The lines are
-rasterized with scaling factor that is as close as possible to the device
-pixels, to be suitable for direct send to the polyline() API call. If PRESCALE
+rasterized with a scaling factor that is as close as possible to the device
+pixels, to be suitable for a call to the polyline() method. If the PRESCALE
 factor is set, it is used instead to premultiply coordinates of arc anchor
 points used to render the lines.
 
-=item points %opt
+=item points
 
-Runs all accumulated commands, and returns rendered set of points, suitable
-for further calls to either C<Prima::Drawable::polyline> or C<Prima::Drawable::fillpoly>
-depending on the C<$opt{fill}> flag.
+Runs all accumulated commands, returns rendered set of points suitable
+for the C<Prima::Drawable::polyline> and C<Prima::Drawable::fillpoly> methods.
 
-=item region MODE=fm::Winding|fm::Overlay, RGNOP=rgnop::Union
+=item region MODE=fm::Winding|fm::Overlay
 
-Creates a region object from polygonal shape. If MODE is set, applies fill mode
-(see L<Prima::Drawable/fillMode> for more); if RGNOP is set, applies region set operation
-(see L<Prima::Region/combine>).
+Creates a region object from the path. If MODE is set, applies fill mode (see
+L<Prima::Drawable/fillMode> for more)
 
 =item stroke
 
@@ -1608,14 +1777,15 @@ Draws a polyline over the path
 
 =item widen %OPTIONS
 
-Expands path into a new path object containing outlines of the original path as
-if drawn with selected line properties. C<lineWidth>, C<lineEnd>, C<lineJoin>,
-C<linePattern> are read from C<%OPTIONS>, or from the attached canvas when
-available. Supports C<miterLimit> option with values from 0 to 20.
+Expands the path into a new path object containing outlines of the original
+path as if drawn with selected line properties. The values of C<lineWidth>,
+C<lineEnd>, C<lineJoin>, and C<linePattern> are read from C<%OPTIONS>, or from
+the attached canvas when available. Supports the C<miterLimit> option with
+values from 0 to 20.
 
-Note: if the intention is to immediately render lines, decrease lineWidth by 1
-(they are 1 pixel wider because paths are built around assumption that pixel size is 0,
-which makes them scalable).
+Note: if the intention is to immediately render non-antialiased lines, decrease
+lineWidth by 1 (they are 1 pixel wider because paths are built around the
+assumption that pixel size is 0, which makes them scalable).
 
 =back
 
@@ -1631,13 +1801,13 @@ as a macro.
 
 =item identity
 
-Returns identity matrix
+Returns the identity matrix
 
 =item matrix_apply @POINTS
 
-Applies current matrix to POINTS, returns the transformed points.
-If @POINTS is a list, returns list; if it is an array reference, returns
-array reference.
+Applies the CTM to POINTS, returns the transformed points.  If @POINTS is a
+list, returns the transformed points as a list; if it is an array reference,
+returns an array reference.
 
 =back
 
