@@ -1,6 +1,7 @@
 package Prima::Drawable::TextBlock;
 use strict;
 use warnings;
+use Prima;
 
 package
     tb;
@@ -138,8 +139,13 @@ sub opcode
 	my $len = $_[0] || 0;
 	my $name = $_[1];
 	$len = 0 if $len < 0;
-	my $op = ++$lastop;
-	$opnames{$name} = $op if defined $name;
+	my $op;
+	if ( defined $name && exists $opnames{$name}) {
+		$op = $opnames{$name};
+	} else {
+		$op = ++$lastop;
+		$opnames{$name} = $op if defined $name;
+	}
 	return $op | (( $len + 1 ) << 16);
 }
 
@@ -189,7 +195,7 @@ sub realize_colors
 
 sub _debug_block
 {
-	my ($b) = @_;
+	my ($b, $text) = @_;
 	print STDERR "FLAGS      : ", (( $$b[BLK_FLAGS] & T_SIZE ) ? "T_SIZE" : ""), (( $$b[BLK_FLAGS] & T_WRAPABLE ) ? "T_WRAPABLE" : ""), "\n";
 	print STDERR "POSITION   : ", $$b[BLK_X] // 'undef', 'x', $$b[BLK_Y] // 'undef', "\n";
 	print STDERR "SIZE       : ", $$b[BLK_WIDTH] // 'undef', 'x', $$b[BLK_HEIGHT] // 'undef', "\n";
@@ -204,7 +210,7 @@ sub _debug_block
 	} elsif ( $color & COLOR_INDEX) {
 		$color = "index(" . ( $color & ~COLOR_INDEX) . ")";
 	} else {
-		$color = sprintf("%06x", $color);
+		$color = sprintf("%06x", $color & 0xffffff);
 	}
 	print STDERR "COLORS     : $color ";
 	$color = $$b[BLK_BACKCOLOR];
@@ -213,9 +219,10 @@ sub _debug_block
 	} elsif ( $color & COLOR_INDEX) {
 		$color = "index(" . ( $color & ~COLOR_INDEX) . ")";
 	} else {
-		$color = sprintf("%06x", $color);
+		$color = sprintf("%06x%s", $color & 0xffffff, ($color & BACKCOLOR_OFF) ? ',transparent' : '');
 	}
 	print STDERR "$color\n";
+	my %opval = reverse %opnames;
 
 	my ($i, $lim) = (BLK_START, scalar @$b);
 	for ( ; $i < $lim; $i += $$b[$i] >> 16) {
@@ -229,7 +236,11 @@ sub _debug_block
 			my $ofs = $$b[ $i + T_OFS];
 			my $len = $$b[ $i + T_LEN];
 			my $wid = $$b[ $i + T_WID] // 'NULL';
-			print STDERR ": OP_TEXT( $ofs $len : $wid )\n";
+			print STDERR ": OP_TEXT( $ofs $len : $wid )";
+			print STDERR ": (",
+				substr( $$text, $$b[BLK_TEXT_OFFSET] + $$b[$i + T_OFS], $$b[$i + T_LEN] ),
+				")" if $text;
+			print STDERR "\n";
 		} elsif ( $cmd == OP_FONT ) {
 			my $mode = $$b[ $i + F_MODE ];
 			my $data = $$b[ $i + F_DATA ];
@@ -257,6 +268,10 @@ sub _debug_block
 				$bk = 'backcolor,';
 				$color &= ~BACKCOLOR_FLAG;
 			}
+			if ( $color & BACKCOLOR_OFF ) {
+				$bk = 'transparent,';
+				$color &= ~BACKCOLOR_OFF;
+			}
 			if ( $color & COLOR_INDEX) {
 				$color = "index(" . ( $color & ~COLOR_INDEX) . ")";
 			} else {
@@ -265,7 +280,7 @@ sub _debug_block
 			print STDERR ": OP_COLOR $bk$color\n";
 		} elsif ( $cmd == OP_TRANSPOSE) {
 			my $x = $$b[ $i + X_X ];
-			my $y = $$b[ $i + X_X ];
+			my $y = $$b[ $i + X_Y ];
 			my $f = $$b[ $i + X_FLAGS ] ? 'EXTEND' : 'TRANSPOSE';
 			print STDERR ": OP_TRANSPOSE $x $y $f\n";
 		} elsif ( $cmd == OP_CODE ) {
@@ -284,8 +299,11 @@ sub _debug_block
 			print STDERR ": OP_MARK $id $x $y\n";
 		} else {
 			my $oplen = $cmd >> 16;
+			$cmd &= 0xffff;
+			$cmd = $opval{$cmd} if defined $opval{$cmd};
 			my @o = ($oplen > 1) ? @$b[ $i + 1 .. $i + $oplen - 1] : ();
 			print STDERR ": OP($cmd) @o\n";
+			last unless $$b[$i] >> 16;
 		}
 	}
 }
@@ -316,16 +334,13 @@ sub walk
 	$commands[ $opnames{$_} & 0xffff ] = $commands{$_} for grep { exists $opnames{$_} } keys %commands;
 	my $ret;
 
-	my ( $text_offset, $f_taint, $font, $c_taint, $paint_state, %save_properties );
+	my ( $text_offset, $f_taint, $font, $c_taint, $paint_state, %save_properties, $gp_context, $want_gp_context );
 
 	# save paint state
 	if ( $trace & TRACE_PAINT_STATE ) {
 		$paint_state = $canvas-> get_paint_state;
-		if ($paint_state) {
-			$save_properties{set_font} = $canvas->get_font if $trace & TRACE_FONTS;
-			if ($trace & TRACE_COLORS) {
-				$save_properties{$_} = $canvas->$_() for qw(color backColor textOpaque);
-			}
+		if ( $paint_state ){
+			$want_gp_context = 1;
 		} else {
 			$canvas-> begin_paint_info;
 		}
@@ -353,12 +368,20 @@ sub walk
 			next unless $$block[$i + T_LEN] > 0;
 
 			if (( $trace & TRACE_FONTS) && ($trace & TRACE_REALIZE) && !$f_taint) {
+				if ( $want_gp_context) {
+					$canvas->graphic_context_push unless $gp_context;
+					$gp_context = 1;
+				}
 				$realize->($state, REALIZE_FONTS);
-				$f_taint = 1;
+				$f_taint    = 1;
 			}
 			if (( $trace & TRACE_COLORS) && ($trace & TRACE_REALIZE) && !$c_taint) {
+				if ( $want_gp_context) {
+					$canvas->graphic_context_push unless $gp_context;
+					$gp_context = 1;
+				}
 				$realize->($state, REALIZE_COLORS);
-				$c_taint = 1;
+				$c_taint   = 1;
 			}
 			$ret = $sub->(
 				@opcode,
@@ -370,12 +393,13 @@ sub walk
 			last if $$semaphore;
 			next;
 		} elsif (($cmd == OP_FONT) && ($trace & TRACE_FONTS)) {
-			if ( $$block[$i + F_MODE] == F_SIZE && $$block[$i + F_DATA] < F_HEIGHT ) {
-				$$state[ $$block[$i + F_MODE]] = $def_fs + $$block[$i + F_DATA];
-			} elsif ( $$block[$i + F_MODE] == F_STYLE ) {
-				$$state[ $$block[$i + F_MODE]] = $$block[$i + F_DATA] | $def_fl;
+			my ($op, $data) = @{$block}[$i + F_MODE, $i + F_DATA];
+			if ( $op == F_SIZE && $data < F_HEIGHT ) {
+				$$state[F_SIZE] = $def_fs + $data;
+			} elsif ( $op == F_STYLE ) {
+				$$state[F_STYLE] = $data | $def_fl;
 			} else {
-				$$state[ $$block[$i + F_MODE]] = $$block[$i + F_DATA];
+				$$state[$op] = $data;
 			}
 			$font = $f_taint = undef;
 		} elsif (($cmd == OP_COLOR) && ($trace & TRACE_COLORS)) {
@@ -392,8 +416,12 @@ sub walk
 			if (($trace & TRACE_FONTS) && ($trace & TRACE_REALIZE)) {
 				if ( $f & X_DIMENSION_FONT_HEIGHT) {
 					unless ( $f_taint) {
+						if ( $want_gp_context) {
+							$canvas->graphic_context_push unless $gp_context;
+							$gp_context = 1;
+						}
 						$realize->($state, REALIZE_FONTS);
-						$f_taint = 1;
+						$f_taint    = 1;
 					}
 					$font //= $canvas-> get_font;
 					$x *= $font-> {height};
@@ -402,11 +430,11 @@ sub walk
 				}
 			}
 			if ( $f & X_DIMENSION_POINT) {
-				$x *= $resolution->[0] / 72;
-				$y *= $resolution->[1] / 72;
+				$x = int($x * $resolution->[0] / 72 + .5);
+				$y = int($y * $resolution->[1] / 72 + .5);
 				$f &= ~X_DIMENSION_POINT;
 			}
-			$ret = $sub->( $x, $y, $f ) if $sub;
+			$ret = $sub->( @opcode, $x, $y, $f ) if $sub;
 			if (!($f & X_EXTEND) && ($trace & TRACE_POSITION)) {
 				$$position[0] += $x;
 				$$position[1] += $y;
@@ -415,12 +443,24 @@ sub walk
 			next;
 		} elsif (( $cmd == OP_CODE) && ($trace & TRACE_PENS) && ($trace & TRACE_REALIZE)) {
 			unless ( $f_taint) {
-				$realize->($state, REALIZE_FONTS);
-				$f_taint = 1;
+				if ($trace & TRACE_FONTS) {
+					if ( $want_gp_context) {
+						$canvas->graphic_context_push unless $gp_context;
+						$gp_context = 1;
+					}
+					$realize->($state, REALIZE_FONTS);
+				}
+				$f_taint  = 1;
 			}
 			unless ( $c_taint) {
-				$realize->($state, REALIZE_COLORS);
-				$c_taint = 1;
+				if ($trace & TRACE_COLORS) {
+					if ( $want_gp_context) {
+						$canvas->graphic_context_push unless $gp_context;
+						$gp_context = 1;
+					}
+					$realize->($state, REALIZE_COLORS);
+				}
+				$c_taint  = 1;
 			}
 		} elsif (( $cmd == OP_MARK) & ( $trace & TRACE_UPDATE_MARK)) {
 			$$block[ $i + MARK_X] = $$position[0];
@@ -438,7 +478,7 @@ sub walk
 	# restore paint state
 	if ( $trace & TRACE_PAINT_STATE ) {
 		if ( $paint_state ) {
-			$canvas->$_( $save_properties{$_} ) for keys %save_properties;
+			$canvas->graphic_context_pop if $gp_context;
 		} else {
 			$canvas->end_paint_info;
 		}
@@ -466,8 +506,10 @@ sub block_wrap
 	my ( @ret, $z, $ptr);
 	my $lastTextOffset = $$b[ BLK_TEXT_OFFSET];
 	my $has_text;
-	my $word_break = $opt{wordBreak};
-	my $wrap_opts  = $word_break ? tw::WordBreak : 0;
+	my $strip_leading_spaces = $opt{stripLeadingSpaces} // 1;
+	my $ignore_wrap_commands = $opt{ignoreWraps} // 0;
+	my $word_break           = $opt{wordBreak};
+	my $wrap_opts            = $word_break ? tw::WordBreak : 0;
 
 	my $newblock = sub
 	{
@@ -502,8 +544,10 @@ sub block_wrap
 		pointer => \$ptr,
 		canvas  => $canvas,
 		state   => $state,
-		trace   => TRACE_REALIZE_PENS,
-		realize => sub { $canvas->font(realize_fonts($subopt{fontmap}, $_[0])) if $_[1] & REALIZE_FONTS },
+		trace   => TRACE_REALIZE_PENS | TRACE_PAINT_STATE,
+		realize => sub {
+			$canvas->font(realize_fonts($subopt{fontmap}, $_[0])) if $_[1] & REALIZE_FONTS;
+		},
 		text    => sub {
 			my ( $ofs, $tlen ) = @_;
 			my $state_key = join('.', @$state[BLK_FONT_ID .. BLK_FONT_STYLE]);
@@ -512,35 +556,41 @@ sub block_wrap
 			$lastTextOffset = $ofs + $tlen unless $can_wrap;
 
 		REWRAP:
-			my $tw  = $canvas-> get_text_shape_width(substr( $$t, $o + $ofs, $tlen), 1, $flags);
+			my $tw  = $canvas-> get_text_shape_width(substr( $$t, $o + $ofs, $tlen), to::AddOverhangs, $flags);
 			my $apx = $state_hash{$state_key}-> {width};
 			if ( $x + $tw + $apx <= $width) {
 				push @$z, OP_TEXT, $ofs, $tlen, $tw;
 				$x += $tw;
 				$has_text = 1;
+			} elsif ( $x + $apx >= $width ) {
+				return if $x == 0;
+				$newblock-> ();
+				goto REWRAP;
 			} elsif ( $can_wrap) {
 				return if $tlen <= 0;
 				my $str = substr( $$t, $o + $ofs, $tlen);
 				my $leadingSpaces = '';
-				if ( $str =~ /^(\s+)/) {
+				if ( $strip_leading_spaces && $str =~ /^(\s+)/) {
 					$leadingSpaces = $1;
 					$str =~ s/^\s+//;
 				}
+				my $shaped = $canvas-> text_shape($str, rtl => $flags);
 				my $l = $canvas-> text_wrap( $str, $width - $apx - $x,
-					tw::ReturnFirstLineLength | tw::BreakSingle | $wrap_opts );
+					tw::ReturnFirstLineLength | tw::BreakSingle | $wrap_opts,
+					8, 0, -1, $shaped || undef);
 				if ( $l > 0) {
 					if ( $has_text) {
 						push @$z, OP_TEXT,
 							$ofs, $l + length $leadingSpaces,
 							$tw = $canvas-> get_text_shape_width(
-								$leadingSpaces . substr( $str, 0, $l), 1,
+								$leadingSpaces . substr( $str, 0, $l), to::AddOverhangs,
 								$flags
 							);
 					} else {
 						push @$z, OP_TEXT,
 							$ofs + length $leadingSpaces, $l,
 							$tw = $canvas-> get_text_shape_width(
-								substr( $str, 0, $l), 1,
+								substr( $str, 0, $l), to::AddOverhangs,
 								$flags
 							);
 						$has_text = 1;
@@ -553,7 +603,7 @@ sub block_wrap
 					if ( $str =~ /^(\s+)/) {
 						$ofs  += length $1;
 						$tlen -= length $1;
-						$x    += $canvas-> get_text_shape_width( $1, 1, $flags);
+						$x    += $canvas-> get_text_shape_width( $1, to::AddOverhangs, $flags);
 						$str =~ s/^\s+//;
 					}
 					goto REWRAP if length $str;
@@ -567,7 +617,7 @@ sub block_wrap
 					# but may be some words can be stripped?
 						goto REWRAP if $ox > 0;
 						if ( $word_break && ($str =~ m/^(\S+)(\s*)/)) {
-							$tw = $canvas-> get_text_shape_width( $1, 1, $flags);
+							$tw = $canvas-> get_text_shape_width( $1, to::AddOverhangs, $flags);
 							push @$z, OP_TEXT, $ofs, length $1, $tw;
 							$has_text = 1;
 							$x += $tw;
@@ -576,8 +626,9 @@ sub block_wrap
 							goto REWRAP;
 						}
 					}
+					my $rr = $x;
 					push @$z, OP_TEXT, $ofs, length($str),
-						$x += $canvas-> get_text_shape_width( $str, 1, $flags);
+						$x += $canvas-> get_text_shape_width( $str, to::AddOverhangs, $flags);
 					$has_text = 1;
 				}
 			} elsif ( $haswrapinfo) { # unwrappable, and cannot be fit - retrace
@@ -592,6 +643,8 @@ sub block_wrap
 			}
 		},
 		wrap => sub {
+			return if $ignore_wrap_commands;
+
 			my $mode = shift;
 			if ( $can_wrap && $mode == WRAP_MODE_OFF) {
 				@wrapret = ( scalar @$z, [ @$state ], $ptr);
@@ -697,6 +750,158 @@ sub block_wrap
 	return @ret;
 }
 
+sub get_text_width_with_overhangs
+{
+	my ( $b, %opt) = @_;
+
+	return $$b[BLK_WIDTH] if !wantarray && defined $$b[BLK_WIDTH];
+
+	my $canvas = $opt{canvas};
+
+	my $last_letter_ofs;
+	walk( $b, %opt,
+		trace     => tb::TRACE_TEXT,
+		text      => sub { $last_letter_ofs = $_[0] },
+	);
+
+	my @xy = (0,0);
+	my $last_c_width;
+	my $first_a_width;
+	walk( $b, %opt,
+		position  => \@xy,
+		trace     => tb::TRACE_REALIZE_FONTS | tb::TRACE_TEXT | tb::TRACE_GEOMETRY |
+				(( $opt{restoreCanvas} // 1) ? tb::TRACE_PAINT_STATE : 0 ),
+		text      => sub {
+			my ($ofs, $len, undef, $t) = @_;
+			my ($whole, $this_c_width);
+			if ( !defined $first_a_width) {
+				my $char = substr( $t, 0, 1 );
+				( $first_a_width, undef, $this_c_width ) = @{ $canvas->get_font_abc(
+					ord($char), ord($char), utf8::is_utf8($t)
+				) };
+				$whole++ if $len == 1;
+			}
+			if ( $ofs == $last_letter_ofs ) {
+				if ( $whole ) {
+					$last_c_width = $this_c_width;
+				} else {
+					my $char = substr( $t, -1, 1 );
+					( undef, undef, $last_c_width ) = @{ $canvas->get_font_abc(ord($char), ord($char), utf8::is_utf8($t)) };
+				}
+			}
+		},
+	);
+	if ( defined $first_a_width ) {
+		$first_a_width = ( $first_a_width < 0 ) ? -$first_a_width : 0;
+		$last_c_width  = ( $last_c_width  < 0 ) ? -$last_c_width : 0;
+	} else {
+		$first_a_width = $last_c_width = 0;
+	}
+
+	return wantarray ?
+		($xy[0], $first_a_width, $last_c_width) :
+		$xy[0] + $first_a_width + $last_c_width;
+}
+
+sub justify_interspace
+{
+	my ($b, %opt) = @_;
+	my ($canvas, $width) = @opt{qw(canvas width)};
+
+	my $curr_width = $$b[BLK_WIDTH] // scalar get_text_width_with_overhangs($b, %opt);
+	return if $curr_width > $opt{width} || $curr_width == 0;
+	my $min_text_to_space_ratio = $opt{max_text_to_space_ratio} // 0.75;
+	return if $curr_width / $width < $min_text_to_space_ratio;
+	my @new = @$b[0 .. BLK_DATA_END];
+	my @breaks;
+	my $n_spaces = 0;
+	my $tt = '';
+	my $space_width;
+	my $got_spaces_at_start;
+	my $combined_width = 0;
+	walk( $b, %opt,
+		trace     => TRACE_TEXT | TRACE_REALIZE_FONTS | TRACE_PAINT_STATE,
+		other     => sub { push @new, @_ },
+		font      => sub {
+			push @new, font(@_);
+			undef $space_width;
+		},
+		text      => sub {
+			my ($ofs, $len, $wid, $t) = @_;
+			unless ($t =~ m/^(\s*)(\S+\s+\S.*?)(\s*)$/) {
+				push @new, text(@_);
+				$combined_width += $wid;
+				return;
+			}
+
+			my ($start, $mid, $end) = ($1, $2, $3);
+			($start, $mid) = ('', "$start$mid") if $got_spaces_at_start;
+			$got_spaces_at_start = 1;
+
+			my @txt;
+			while ( 1 ) {
+				my $tx;
+				if ( $mid =~ m/\G(\s+)/gcs) {
+					my $l = length($1);
+					$ofs += $l;
+					$space_width //= $canvas->get_text_shape_width(' ');
+					push @txt, undef, undef, $l * $space_width;
+					$n_spaces++;
+					$combined_width += $l * $space_width;
+					next;
+				} elsif ( $mid =~ m/\G$/gcs) {
+					last;
+				} elsif ($mid =~ m/\G^(\S+)/gcs) {
+					$tx = "$start$1";
+				} elsif ( $mid =~ m/\G(\S+)$/gcs) {
+					$tx = "$1$end";
+				} elsif ( $mid =~ m/\G(\S+)/gcs) {
+					$tx = $1;
+				}
+#				$tt .= "$tx ";
+
+				my $l = length($tx);
+				my $tw = $canvas->get_text_shape_width($tx);
+				$combined_width += $tw;
+				push @txt, $ofs, $l, $tw;
+				$ofs += $l;
+			}
+			push @breaks, [ scalar(@new), \@txt ];
+		},
+	);
+	return unless $n_spaces;
+	return if $combined_width >= $width;
+
+	my $avg_space_incr  = ($width - $combined_width) / $n_spaces;
+	my ($curr, $last_incr) = (0,0);
+	my $ff = $canvas->font;
+	my $fh = $ff->{height} + $ff->{externalLeading};
+	my $dh = $$b[BLK_APERTURE_Y] - $ff->{descent};
+
+	for ( my $i = $#breaks; $i >= 0; $i--) {
+		my ( $at, $txt ) = @{ $breaks[$i] };
+		my @blk;
+		for ( my $j = 0; $j < @$txt; $j += 3) {
+			if ( defined $$txt[$j] ) {
+				push @blk, text(@$txt[$j .. $j+2] );
+			} else {
+				$curr += $avg_space_incr;
+				my $incr = int( $curr - $last_incr );
+				$last_incr += $incr;
+				push @blk, moveto(0, $dh) if $dh != 0;
+				push @blk, extend($$txt[$j + 2] + $incr, $fh);
+				push @blk, moveto($$txt[$j + 2] + $incr, -$dh);
+			}
+		}
+		splice( @new, $at, 0, @blk);
+	}
+
+	$new[BLK_WIDTH] = $width;
+
+	return \@new;
+}
+
+
 package Prima::Drawable::TextBlock;
 
 sub new
@@ -709,7 +914,7 @@ sub new
 		direction     => 0,
 		fontmap       => [],
 		colormap      => [],
-		text          => '',
+		textRef       => \'',
 		textDirection => 0,
 		block         => undef,
 		resolution    => [72,72],
@@ -720,10 +925,16 @@ sub new
 }
 
 eval "sub $_ { \$#_ ? \$_[0]->{$_} = \$_[1] : \$_[0]->{$_}}" for qw(
-	fontmap colormap block text resolution direction
+	fontmap colormap block textRef resolution direction
 	baseFontSize baseFontStyle restoreCanvas textDirection
 );
 
+sub text
+{
+	return ${ $_[0]->{textRef} } unless $#_;
+	$_[0]->{textRef} = \ $_[1];
+}
+
 sub acquire {}
 
 sub calculate_dimensions
@@ -745,7 +956,7 @@ sub calculate_dimensions
 			my ( undef, undef, undef, $text ) = @_;
 			$b-> [ $ptr + tb::T_WID ] = $canvas->get_text_shape_width(
 				$text,
-				$self->{textDirection} ? to::RTL : 0
+				($self->{textDirection} ? to::RTL : 0) | to::AddOverhangs,
 			);
 
 			my $f = $canvas->get_font;
@@ -774,7 +985,7 @@ sub walk_options
 {
 	my $self = shift;
 	return
-		textPtr => \ $self->{text},
+		textPtr => $self->{textRef},
 		( map { ($_ , $self->{$_}) } qw(fontmap colormap resolution baseFontSize baseFontSize) ),
 		;
 }
@@ -836,43 +1047,14 @@ sub text_out
 	return not $semaphore;
 }
 
-sub get_text_width_with_overhangs
-{
-	my ($self, $canvas) = @_;
-	my $first_a_width;
-	my $last_c_width;
-	my @xy = (0,0);
-	tb::walk( $self->{block}, $self-> walk_options,
-		position  => \@xy,
-		trace     => tb::TRACE_GEOMETRY | tb::TRACE_REALIZE | tb::TRACE_TEXT |
-				( $self-> {restoreCanvas} ? tb::TRACE_PAINT_STATE : 0 ),
-		canvas    => $canvas,
-		text      => sub {
-			my $t = pop;
-			if ( !defined $first_a_width) {
-				my $char = substr( $t, 0, 1 );
-				( $first_a_width ) = @{ $canvas->get_font_abc(ord($char), ord($char), utf8::is_utf8($t)) };
-			}
-			my $char = substr( $t, -1, 1 );
-			( undef, undef, $last_c_width ) = @{ $canvas->get_font_abc(ord($char), ord($char), utf8::is_utf8($t)) };
-		},
-	);
-	return (0,0,0) unless defined $first_a_width;
-	$first_a_width = ( $first_a_width < 0 ) ? -$first_a_width : 0;
-	$last_c_width  = ( $last_c_width  < 0 ) ? -$last_c_width : 0;
-	return ($xy[0], $first_a_width, $last_c_width);
-}
-
 sub get_text_width
 {
 	my ( $self, $canvas, $add_overhangs) = @_;
 
 	$self->acquire($canvas, font => 1, dimensions => 1);
 
-	if ( $add_overhangs ) {
-		my ( $width, $a, $c) = $self-> get_text_width_with_overhangs($canvas);
-		return $width + $a + $c;
-	}
+	return scalar tb::get_text_width_with_overhangs( $self->{block}, $self-> walk_options )
+		if $add_overhangs;
 
 	my @xy = (0,0);
 	tb::walk( $self->{block}, $self-> walk_options,
@@ -921,11 +1103,12 @@ sub text_wrap
 {
 	my ( $self, $canvas, $width, $opt, $indent) = @_;
 	$opt //= tw::Default;
+	$width = 2_000_000 if $width < 0; 
 
 	# Ignored options: ExpandTabs, CalcTabs .
 
 	# first, we don't return chunks, period. That's too messy.
-	return $canvas-> text_wrap( $self-> {text}, $width, $opt, $indent)
+	return $canvas-> text_wrap( ${$self-> {textRef}}, $width, $opt, $indent)
 		if $opt & tw::ReturnChunks;
 
 	$self->acquire($canvas, font => 1);
@@ -991,6 +1174,81 @@ sub height
 	return $self->{block}->[tb::BLK_HEIGHT];
 }
 
+package Prima::Drawable::PolyTextBlock;
+use base qw(Prima::Drawable::TextBlock);
+
+
+sub new
+{
+	my ($class, %opt) = @_;
+	my $self = $class->SUPER::new(%opt);
+	$self->{blocks} //= [];
+	return $self;
+}
+
+sub blocks { $_[0]->{blocks} }
+
+sub for_blocks
+{
+	my ( $self, $canvas, $cb ) = @_;
+
+	my $ps = $canvas ? $canvas->get_paint_state : ps::Enabled;
+	$canvas->begin_paint_info if $ps == ps::Disabled;
+
+	for my $b ( @{ $self->{blocks} }) {
+		$self->{block} = $b;
+		$cb->(@_);
+	}
+
+	$canvas->end_paint_info if $ps == ps::Disabled;
+}
+
+sub calculate_dimensions
+{
+	my ( $self, $canvas ) = @_;
+	$self-> for_blocks( $canvas, sub {
+		$self-> SUPER::calculate_dimensions( $canvas );
+	} );
+}
+
+sub text_out
+{
+	my ($self, $canvas, $x, $y) = @_;
+	$self-> for_blocks( $canvas, sub {
+		my $b = $self->{block};
+		$self-> SUPER::text_out( $canvas, $x, $y);
+		$y -= $$b[tb::BLK_HEIGHT];
+	});
+}
+
+sub get_text_width
+{
+	my ( $self, $canvas, $add_overhangs) = @_;
+
+	my $max = 0;
+	$self-> for_blocks( $canvas, sub {
+		my $x = $self-> SUPER::get_text_width( $canvas, $add_overhangs );
+		$max = $x if $max < $x;
+	} );
+	return $max;
+}
+
+sub get_text_box
+{
+	# XXX unimplemented - todo
+}
+
+sub text_wrap
+{
+	my ( $self, $canvas, $width, $opt, $indent) = @_;
+	my @ret;
+	$self-> for_blocks( $canvas, sub {
+		my $x = $self-> SUPER::text_wrap( $canvas, $width, $opt, $indent);
+		push @ret, @$x;
+	} );
+	return \@ret;
+}
+
 1;
 
 =pod
@@ -1004,7 +1262,7 @@ Prima::Drawable::TextBlock - rich text representation
 =head2 Block header
 
 A block's fixed header consists of C<tb::BLK_START - 1> integer scalars,
-each of those is accessible via the corresponding C<tb::BLK_XXX> constant.
+each of which is accessible via the corresponding C<tb::BLK_XXX> constant.
 The constants are separated into two logical groups:
 
 	BLK_FLAGS
@@ -1024,23 +1282,24 @@ and
 	BLK_COLOR
 	BLK_BACKCOLOR
 
-The second group is enclosed in C<tb::BLK_DATA_START> - C<tb::BLK_DATA_END>
-range, like the whole header is contained in 0 - C<tb::BLK_START - 1> range.
-This is done for the backward compatibility, if the future development changes
-the length of the header.
+The first group defines the offset constants that are used to address the
+values in the block header; the constants lie in the 0 - C<tb::BLK_START - 1>
+range.  The second group values line in the C<tb::BLK_DATA_START> -
+C<tb::BLK_DATA_END> range.  This is done for eventual backward compatibility,
+if the future development changes the length of the header.
 
-The first group fields define the text block dimension, aperture position
-and text offset ( remember, the text is stored as one big chunk ). The second
+The fields from the first group define the text block dimension, aperture position,
+and text offset ( remember, the text is stored as one big chunk ). The second group
 defines the initial color and font settings. Prima::TextView needs all fields
-of every block to be initialized before displaying. L<block_wrap> method
-can be used for automated assigning of these fields.
+of every block to be initialized before displaying. The L<block_wrap> method
+can be used for the automated assigning of these fields.
 
 =head2 Block parameters
 
-The scalars, beginning from C<tb::BLK_START>, represent the commands to the
-renderer.  These commands have their own parameters, that follow the command.
-The length of a command is high 16-bit word of the command. The basic command
-set includes C<OP_TEXT>, C<OP_COLOR>, C<OP_FONT>, C<OP_TRANSPOSE>, and
+The scalars after C<tb::BLK_START> encode the commands to the block renderer.
+These commands have their own parameters which follow the command.  The length
+of the command is encoded in the high 16-bit word of the command. The basic
+command set includes C<OP_TEXT>, C<OP_COLOR>, C<OP_FONT>, C<OP_TRANSPOSE>, and
 C<OP_CODE>.  The additional codes are C<OP_WRAP> and C<OP_MARK>, not used in
 drawing but are special commands to L<block_wrap>.
 
@@ -1048,122 +1307,125 @@ drawing but are special commands to L<block_wrap>.
 
 =item OP_TEXT - TEXT_OFFSET, TEXT_LENGTH, TEXT_WIDTH
 
-C<OP_TEXT> commands to draw a string, from offset C<tb::BLK_TEXT_OFFSET + TEXT_OFFSET>,
-with a length TEXT_LENGTH. The third parameter TEXT_WIDTH contains the width of the text
-in pixels. Such the two-part offset scheme is made for simplification of an imaginary code,
-that would alter ( insert to, or delete part of ) the big text chunk; the updating procedure
-would not need to traverse all commands, but just the block headers.
+C<OP_TEXT> commands to draw a string, from the offset C<tb::BLK_TEXT_OFFSET +
+TEXT_OFFSET>, with the length TEXT_LENGTH. The third parameter TEXT_WIDTH
+contains the width of the text in pixels. The scheme is made for simplification
+of an imaginary code, that would alter ( insert to, or delete part of ) the
+text; the updating procedure would not need to traverse all commands in all
+blocks, but only the block headers.
 
 Relative to: C<tb::BLK_TEXT_OFFSET>
 
 =item OP_COLOR - COLOR
 
 C<OP_COLOR> sets foreground or background color. To set the background,
-COLOR must be or-ed with C<tb::BACKCOLOR_FLAG> value. In addition to the
-two toolkit supported color values ( RRGGBB and system color index ),
-COLOR can also be or-ed with C<tb::COLOR_INDEX> flags, in such case it is
-an index in C<::colormap> property array.
+COLOR must be or-ed with the C<tb::BACKCOLOR_FLAG> value. In addition to the
+two toolkit-supported color values ( RRGGBB and system color index ),
+COLOR can also be or-ed with the C<tb::COLOR_INDEX> flag, in such case it is
+treated an index in the C<::colormap> property array.
 
 Relative to: C<tb::BLK_COLOR>, C<tb::BLK_BACKCOLOR>.
 
 =item OP_FONT - KEY, VALUE
 
-As the font is a complex property, that itself includes font name, size,
-direction, etc keys, C<OP_FONT> KEY represents one of the three
-parameters - C<tb::F_ID>, C<tb::F_SIZE>, C<tb::F_STYLE>. All three
-have different VALUE meaning.
+As a font is a complex property which includes font name, size, direction, etc
+fields, the C<OP_FONT> KEY represents one of the three parameters -
+C<tb::F_ID>, C<tb::F_SIZE>, C<tb::F_STYLE>. All three have different VALUE
+meanings.
 
-Relative to: C<tb::BLK_FONT_ID>, C<tb::BLK_FONT_SIZE>, C<tb::BLK_FONT_STYLE>.
+Relates to: C<tb::BLK_FONT_ID>, C<tb::BLK_FONT_SIZE>, C<tb::BLK_FONT_STYLE>.
 
 =over
 
 =item F_STYLE
 
-Contains a combination of C<fs::XXX> constants, such as C<fs::Bold>, C<fs::Italic> etc.
+Contains a combination of the C<fs::XXX> constants, such as C<fs::Bold>, C<fs::Italic> etc.
 
 Default value: 0
 
 =item F_SIZE
 
-Contains the relative font size. The size is relative to the current widget's font
-size. As such, 0 is a default value, and -2 is the widget's default font decreased by
+Contains the relative font size. The size is relative to the current font
+size. As such, 0 is a default value, and -2 is the default font decreased by
 2 points. Prima::TextView provides no range checking ( but the toolkit does ), so
 while it is o.k. to set the negative C<F_SIZE> values larger than the default font size,
-one must be vary when relying on the combined font size value .
+one must be careful when relying on the combined font size value .
 
-If C<F_SIZE> value is added to a C<F_HEIGHT> constant, then it is treated as a font height
-in pixels rather than font size in points. The macros for these opcodes are named respectively
-C<tb::fontSize> and C<tb::fontHeight>, while the opcode is the same.
+If the C<F_SIZE> value is added to the C<F_HEIGHT> constant, then it is treated
+as font height in pixels rather than font size in points. The macros for these
+opcodes are named respectively C<tb::fontSize> and C<tb::fontHeight>, while the
+opcode is the same.
 
 =item F_ID
 
-All other font properties are collected under an 'ID'. ID is a index in
+All other font properties are collected under an 'ID'. ID is an index in
 the C<::fontPalette> property array, which contains font hashes with the other
-font keys initialized - name, encoding, and pitch. These three are minimal required
-set, and the other font keys can be also selected.
+font keys initialized - name, encoding, and pitch. These three fields are required
+to be defined in the font hash; the other font fields are optional.
 
 =back
 
 =item OP_TRANSPOSE X, Y, FLAGS
 
-Contains a mark for an empty space. The space is extended to the relative coordinates (X,Y),
-so the block extension algorithms take this opcode in the account. If FLAGS does not contain
-C<tb::X_EXTEND>, then in addition to the block expansion, current coordinate is also
-moved to (X,Y). In this regard, C<(OP_TRANSPOSE,0,0,0)> and C<(OP_TRANSPOSE,0,0,X_EXTEND)> are
-identical and are empty operators.
+Contains a mark for an empty space. The space is extended to the relative
+coordinates (X,Y), so the block extension algorithms take this opcode into
+account. If FLAGS does not contain C<tb::X_EXTEND>, then in addition to the
+block expansion, the current coordinate is also moved to (X,Y). 
+C<(OP_TRANSPOSE,0,0,0)> and C<(OP_TRANSPOSE,0,0,X_EXTEND)> are identical and
+are empty operators.
 
-There are formatting-only flags,in effect with L<block_wrap> function.
-C<X_DIMENSION_FONT_HEIGHT> indicates that (X,Y) values must be multiplied to
+The C<X_DIMENSION_FONT_HEIGHT> flag indicates that (X,Y) values must be multiplied by
 the current font height.  Another flag C<X_DIMENSION_POINT> does the same but
-multiplies by current value of L<resolution> property divided by 72 (
-basically, treats X and Y not as pixel but point values).
+multiplies by the current value of the L<resolution> property divided by 72 (
+treats X and Y not as pixel but as point values).
 
-C<OP_TRANSPOSE> can be used for customized graphics, in conjunction with C<OP_CODE>
-to assign a space, so the rendering
-algorithms do not need to be re-written every time the new graphic is invented. As
-an example, see how L<Prima::PodView> deals with the images.
+C<OP_TRANSPOSE> can be used for customized graphics, in conjunction with
+C<OP_CODE> to assign a space, so the rendering algorithms do not need to be
+rewritten every time a new graphic is invented. For example, see how
+L<Prima::PodView> implements images and bullet points.
 
 =item OP_CODE - SUB, PARAMETER
 
 Contains a custom code pointer SUB with a parameter PARAMETER, passed when
-a block is about to be drawn. SUB is called with the following format:
+the block is about to be drawn. SUB is called with the following format:
 
 	( $widget, $canvas, $text_block, $font_and_color_state, $x, $y, $parameter);
 
 $font_and_color_state ( or $state, through the code ) contains the state of
-font and color commands in effect, and is changed as the rendering algorithm advances through a block.
-The format of the state is the same as of text block, so one may notice that for readability
-F_ID, F_SIZE, F_STYLE constants are paired to BLK_FONT_ID, BLK_FONT_SIZE and BLK_FONT_STYLE.
+font and color commands in effect, and is changed as the rendering algorithm
+advances through the block.  The format of the state is the same as of the text
+block, and the F_ID, F_SIZE, the F_STYLE constants are the same as BLK_FONT_ID,
+BLK_FONT_SIZE, and BLK_FONT_STYLE.
 
-The SUB code is executed only when the block is about to draw.
+The SUB code is executed only when the block is about to be drawn.
 
 =item OP_WRAP mode
 
-C<OP_WRAP> is only in effect in L<block_wrap> method. C<mode> is a flag,
+C<OP_WRAP> is only used in the L<block_wrap> method. C<mode> is a flag,
 selecting the wrapping command.
 
    WRAP_MODE_ON   - default, block commands can be wrapped
    WRAP_MODE_OFF  - cancels WRAP_MODE_ON, commands cannot be wrapped
-   WRAP_IMMEDIATE - proceed with immediate wrapping, unless ignoreImmediateWrap options is set
+   WRAP_IMMEDIATE - proceed with immediate wrapping, unless the ignoreImmediateWrap option is set
 
 L<block_wrap> does not support stacking for the wrap commands, so the
-C<(OP_WRAP,WRAP_MODE_ON,OP_WRAP,WRAP_MODE_ON,OP_WRAP,WRAP_MODE_OFF)> has same
-effect as C<(OP_WRAP,WRAP_MODE_OFF)>. If C<mode> is WRAP_MODE_ON, wrapping is
-disabled - all following commands treated an non-wrapable until
-C<(OP_WRAP,WRAP_MODE_OFF)> is met.
+C<(OP_WRAP,WRAP_MODE_ON,OP_WRAP,WRAP_MODE_ON,OP_WRAP,WRAP_MODE_OFF)> command
+sequence has the same effect as the C<(OP_WRAP,WRAP_MODE_OFF)> sequence. If C<mode> is
+WRAP_MODE_ON, wrapping is disabled - all following commands are treated as
+non-wrappable until the C<(OP_WRAP,WRAP_MODE_OFF)> command sequence is met.
 
 =item OP_MARK PARAMETER, X, Y
 
-C<OP_MARK> is only in effect in L<block_wrap> method and is a user command.
-L<block_wrap> only sets (!) X and Y to the current coordinates when the command is met.
-Thus, C<OP_MARK> can be used for arbitrary reasons, easy marking the geometrical positions
-that undergo the block wrapping.
+C<OP_MARK> is only in effect in the L<block_wrap> method and is a user command.
+L<block_wrap> only sets (!) X and Y to the current coordinates when the command
+is met.  Thus, C<OP_MARK> can be used for arbitrary reasons, for example for
+saving the geometrical positions during the block wrapping.
 
 =back
 
-As can be noticed, these opcodes are far not enough for the full-weight rich text
-viewer. However, the new opcodes can be created using C<tb::opcode>, that accepts
-the opcode length and returns the new opcode value.
+These opcodes are far not enough for the full-weight rich text viewer. However,
+the new opcodes can be created using C<tb::opcode>, which accepts the opcode
+length and returns the new opcode value.
 
 =head2 Rendering methods
 
@@ -1171,35 +1433,41 @@ the opcode length and returns the new opcode value.
 
 =item block_wrap %OPTIONS
 
-C<block_wrap> wraps a block into a given width. It returns one or more text
-blocks with fully assigned headers. The returned blocks are located one below
+C<block_wrap> wraps a block into a given width in pixels. It returns one or more text
+blocks with fully formed headers. The returned blocks are located one below
 another, providing an illusion that the text itself is wrapped.  It does not
-only traverses the opcodes and sees if the command fit or not in the given
+only traverse the opcodes and sees if the command fits in the given
 width; it also splits the text strings if these do not fit.
 
-By default the wrapping can occur either on a command boundary or by the spaces
+By default, the wrapping can occur either on a command boundary or by the spaces
 or tab characters in the text strings. The unsolicited wrapping can be
-prevented by using C<OP_WRAP> command brackets. The commands inside these
-brackets are not wrapped; C<OP_WRAP> commands are removed from the output
+prevented by using the C<OP_WRAP> command brackets. The commands inside these
+brackets are not wrapped; the C<OP_WRAP> commands are removed from the resulting
 blocks.
 
-In general, C<block_wrap> copies all commands and their parameters as is, ( as
-it is supposed to do ), but some commands are treated specially:
+C<block_wrap> copies all commands and their parameters as is, except the following:
 
 - C<OP_TEXT>'s third parameter, C<TEXT_WIDTH>, is disregarded, and is recalculated for every
-C<OP_TEXT> met.
+C<OP_TEXT> command.
 
-- If C<OP_TRANSPOSE>'s third parameter, C<X_FLAGS> contains C<X_DIMENSION_FONT_HEIGHT> flag,
-the command coordinates X and Y are multiplied to the current font height and the flag is
-cleared in the output block.
+- If C<OP_TRANSPOSE>'s third parameter, C<X_FLAGS> contains the
+C<X_DIMENSION_FONT_HEIGHT> flag, the command coordinates X and Y are
+multiplied to the current font height, and the flag is cleared in the output
+block. The C<X_DIMENSION_PIXEL> has a similar effect but the coordinates are
+multiplied by the current resolution divided by 72.
 
-- C<OP_MARK>'s second and third parameters assigned to the current (X,Y) coordinates.
+- C<OP_MARK>'s second and third parameters are assigned to the current (X,Y) coordinates.
 
-- C<OP_WRAP> removed from the output.
+- C<OP_WRAP> is removed from the output.
+
+=item justify_interspace %OPTIONS
+
+Uses C<$OPTIONS{width}> and C<$OPTIONS{min_text_to_space_ratio}> to try to make
+inter-word spacing. Returns new block if successful, undef otherwise.
 
 =item walk BLOCK, %OPTIONS
 
-Cycles through block opcodes, calls supplied callbacks on each.
+Cycles through the block opcodes, calls supplied callbacks on each.
 
 =back
 
