@@ -993,9 +993,49 @@ sub read_jpeg
 		exists $i->{extras}->{appdata} and
 		$data = $i->{extras}->{appdata}->[1];
 
-	return ( $2, "XMP data" )
-		if $data =~ /^($XMP_HEADER)(.*)/s;
+	if (ref($data) ne 'ARRAY') {
+		return { "XMP data" => $2 }
+			if $data =~ /^($XMP_HEADER)(.*)/s;
+		return parse_datum( $class, $data, %opt );
+	} else {
+		my %res;
+		for my $datum ( @$data ) {
+			if ($datum =~ /^($XMP_HEADER)(.*)/s) {
+				$res{"XMP data"} = $2;
+			} else {
+				my ($r, $e) = parse_datum( $class, $datum, %opt );
+				return ($r, $e) unless ref $r;
+				%res = (%res, %$r);
+			}
+		}
+		return \%res;
+	}
+}
 
+sub read_jxl
+{
+	my ($class, $i, %opt) = @_;
+
+	my $data;
+	return (undef, "no jxl exif data") unless
+		($i->codec // '') eq 'JXL' and
+		exists $i->{extras}->{exif} and
+		$data = $i->{extras}->{exif};
+
+	if ( $data =~ /^.{4}Exif/) {
+		$data = substr($data,4);
+	} elsif ( $data =~ /^\x{00}\x{00}/ ) {
+		$data = 'Exif'.substr($data,2);
+	} else {
+		return (undef, "no jxl exif data");
+	}
+
+	return parse_datum( $class, $data, %opt);
+}
+
+sub parse_datum
+{
+	my ( $class, $data, %opt ) = @_;
 	my ( $res, $error) = $class->parse($data);
 	if ( ref($res) ) {
 		if ( $res->{thumbnail} && $opt{load_thumbnail} ) {
@@ -1024,6 +1064,8 @@ sub read_extras
 	my $c = $i->codec // '';
 	if ( $c eq 'JPEG') {
 		return $class->read_jpeg($i, %opt);
+	} elsif ( $c eq 'JXL') {
+		return $class->read_jxl($i, %opt);
 	} else {
 		return (undef, "not supported");
 	}
@@ -1037,6 +1079,16 @@ sub write_jpeg
 
 	my $e = $i->{extras};
 	my %data = %$data;
+	my $xmp;
+
+	if ( exists $data{"XMP data"}) {
+		$xmp = delete $data{"XMP data"};
+		unless ( keys %data ) {
+			$i->{extras}->{appdata}->[1] = $xmp;
+			return 1;
+		}
+	}
+
 	if ( $data{thumbnail} && ref($data{thumbnail})) {
 		my $t = '';
 		open my $f, ">", \$t;
@@ -1059,10 +1111,26 @@ sub write_jpeg
 
 	my ($compiled, $error) = $class->compile(\%data);
 	return (undef, $error) if !defined $compiled;
+
+	$compiled = [$compiled, $xmp] if defined $xmp;
+
 	$i->{extras}->{appdata}->[1] = $compiled;
 	return 1;
 }
 
+sub write_jxl
+{
+	my ($class, $i, $data) = @_;
+
+	return undef, "jpeg-xl is not supported" unless $i->codec('JXL');
+
+	my ($compiled, $error) = $class->compile($data);
+	return (undef, $error) if !defined $compiled;
+	$compiled = substr($compiled,4);
+	$i->{extras}->{exif} = $compiled;
+	return 1;
+}
+
 sub supported_ifds { qw(image photo gpsinfo) }
 
 sub revtags
@@ -1093,6 +1161,8 @@ sub write_extras
 
 	if ( $c eq 'JPEG' ) {
 		return $class->write_jpeg($i, $data);
+	} elsif ( $c eq 'JXL') {
+		return $class->write_jxl($i, $data);
 	} else {
 		return (undef, "not supported");
 	}
@@ -1109,8 +1179,7 @@ Prima::Image::Exif - manipulate Exif records
 =head1 DESCRIPTION
 
 The module allows to parse and create Exif records. The records
-could be read from JPEG files, and stored in these using the
-extra appdata hash field.
+could be read from JPEG and JXL files, and prepared to be stored there.
 
 =head1 SYNOPSIS
 
@@ -1125,9 +1194,9 @@ extra appdata hash field.
 		tag_as_string  => 1
 	);
 
-	if ( $error eq 'XMP data' && defined $data ) {
+	if ( $data && exists $data->{'XMP data') {
 		require XML::LibXML;
-		my $xml = XML::LibXML->load_xml(string => $data);
+		my $xml = XML::LibXML->load_xml(string => $data->{'XMP data'});
 		for my $node ($xml->findnodes('//*')) {
 			my @p = $node->childNodes;
 			next unless @p == 1;
@@ -1137,7 +1206,8 @@ extra appdata hash field.
 			$p =~ s{^\.}{};
 			print "$p: $p[0]\n";
 		}
-		($data, $error) = ({}, undef);
+		delete $data->{'XMP data'};
+		undef $error;
 	}
 
 	die "cannot read exif: $error\n" if defined $error;
@@ -1233,7 +1303,7 @@ L<Prima::image-load>
 The module does not provide comprehensive support for tags and their values.
 For a more thorough dive see these modules: L<Image::ExifTool>, L<Image::EXIF>.
 
-The module doesn't parse XMP records but warns if get supplied these.
+The module doesn't parse XMP records but returns them as raw strings.
 The XMP records are easily parsed by any XML parser, f ex L<XML::LibXML>.
 
 =head1 AUTHOR
