@@ -78,6 +78,7 @@ sub change_transform
 		$ps->[0] - $pm->[2] - $pm->[0],
 		$ps->[1] - $pm->[3] - $pm->[1]
 	);
+	@pm[0,1] = (0,0) if $self->{lock_margins};
 
 	my ($doClip, @cr);
 	my $rg = $self-> region;
@@ -95,7 +96,7 @@ sub change_transform
 	$self-> emit_content('Q') unless $gsave;
 	$self-> emit_content('q');
 
-	$self-> emit_content("h @pm re W n");
+	$self-> emit_content("@pm re W n");
 
 	my $m = $self-> get_matrix;
 	if ( $self->{reversed}) {
@@ -353,7 +354,7 @@ sub emit_new_dummy_object
 sub begin_doc
 {
 	my ( $self, $docName) = @_;
-	return 0 if $self-> get_paint_state;
+	$@ = 'Bad object', return 0 if $self-> get_paint_state;
 
 	$self-> {ps_data}  = '';
 	$self-> {can_draw} = 1;
@@ -405,13 +406,13 @@ ROOT
 	$self-> {all_alphas}    = {};
 	unless ($self-> {page_content} = $self->new_file_obj) {
 		$self-> abort_doc;
-		return 0;
+		$@ = 'Cannot create temp file', return 0;
 	}
 
 	$self-> {changed} = { map { $_ => 0 } qw(
 		fill lineEnd linePattern lineWidth lineJoin miterLimit font)};
 
-	$self-> SUPER::begin_paint;
+	$self-> {paint_state} = ps::Enabled;
 	$self-> save_state;
 
 	$self-> {delay} = 1;
@@ -497,8 +498,8 @@ sub abort_doc
 	my $self = $_[0];
 	return unless $self-> {can_draw};
 	$self-> {can_draw} = 0;
-	$self-> SUPER::end_paint;
-	$self-> restore_state;
+	delete $self->{paint_state};
+	$self-> abandon_state;
 	delete $self-> {$_} for
 		qw (save_state ps_data changed );
 }
@@ -509,7 +510,7 @@ sub end_paint   {        $_[0]-> abort_doc; }
 sub end_doc
 {
 	my $self = $_[0];
-	return 0 unless $self-> {can_draw};
+	$@ = 'Bad object', return 0 unless $self-> {can_draw};
 	$self-> end_page;
 
 	my $pages = scalar @{ $self->{pages} };
@@ -657,7 +658,7 @@ TRAILER
 	$self->{ps_data} = '';
 
 	$self-> {can_draw} = 0;
-	$self-> SUPER::end_paint;
+	delete $self->{paint_state};
 	$self-> restore_state;
 	delete $self-> {$_} for
 		qw (save_state changed ps_data glyph_keeper glyph_font);
@@ -849,6 +850,7 @@ sub graphic_context_push
 	my $self = shift;
 	return 0 unless $self->SUPER::graphic_context_push;
 	$self->emit_content('q');
+	$self->{lock_margins}++;
 	return 1;
 }
 
@@ -857,6 +859,7 @@ sub graphic_context_pop
 	my $self = shift;
 	$self->emit_content('Q');
 	return unless $self->SUPER::graphic_context_pop;
+	$self->{lock_margins}--;
 	return 1;
 }
 
@@ -952,7 +955,7 @@ sub text_out_outline
 
 sub glyph_out_outline
 {
-	my ( $self, $text, $from, $len ) = @_;
+	my ( $self, $text, $from, $len) = @_;
 
 	my $glyphs     = $text-> glyphs;
 	my $indexes    = $text-> indexes;
@@ -961,9 +964,8 @@ sub glyph_out_outline
 	my $fonts      = $text-> fonts;
 	my $plaintext  = $text-> [Prima::Drawable::Glyphs::CUSTOM()];
 	my @ix_lengths = defined($plaintext) ? $text-> index_lengths : ();
-	my $adv        = 0;
 	my $canvas     = $self->glyph_canvas;
-	my $resolution = 72.27 / $self->{resolution}->[0];
+	my $resolution = 72.0 / $self->{resolution}->[0];
 	my $keeper     = $self->{glyph_keeper};
 	my $font       = $self->{glyph_font};
 	my $div        = $self->{font_scale};
@@ -972,13 +974,10 @@ sub glyph_out_outline
 	$len += $from;
 	my $emit = '';
 	my $fid  = 0;
-	my $ff = $canvas->font;
 	my $curr_subfont = -1;
-	my ($x, $y) = (0,0);
+	my @multi_tj;
 	for ( my $i = $from; $i < $len; $i++) {
-		my $advance;
 		my $glyph     = $glyphs->[$i];
-		my ($x2, $y2) = ($adv, 0);
 		my $nfid = $fonts ? $fonts->[$i] : 0;
 		if ( $nfid != $fid ) {
 			my $newfont;
@@ -1004,26 +1003,17 @@ sub glyph_out_outline
 			$curr_subfont = $subfont;
 			my $xid = $self-> {all_fonts}-> {$font}-> {xids}-> [ $subfont ] //= $self->new_dummy_obj;
 			$self->{page_fonts}->{$xid} //= 1;
+			if ( @multi_tj ) {
+				$emit .= "(".join('',@multi_tj).") Tj\n";
+				@multi_tj = ();
+			}
 			$emit .= "/F$xid $self->{font}->{size} Tf\n";
 		}
-		if ( $advances) {
-			$advance = $advances->[$i];
-			$x2 += $positions->[$i*2];
-			$y2 += $positions->[$i*2 + 1];
-		} else {
-			my $xr = $canvas->get_font_abc($glyph, $glyph, to::Glyphs);
-			$advance = ($$xr[0] + $$xr[1] + $$xr[2]) * $div;
-		}
-		$adv += $advance;
-		($x2, $y2) = map { int( $_ * 100 + 0.5) / 100 } $self->pixel2point($x2, $y2);
-		my $dx = $x2 - $x;
-		my $dy = $y2 - $y;
-		if  ($dx != 0 || $dy != 0) {
-			float_inplace($dx, $dy);
-			$emit .= "$dx $dy Td ";
-		}
-		($x, $y) = ($x2, $y2);
-		$emit .= sprintf "<%02x> Tj\n", $gid if defined $gid;
+		push @multi_tj, sprintf("\\%03o", $gid) if defined $gid;
+	}
+	if ( @multi_tj ) {
+		$emit .= "(".join('',@multi_tj).") Tj\n";
+		@multi_tj = ();
 	}
 
 	if ($restore_font) {
@@ -1249,6 +1239,8 @@ sub put_image_indirect
 {
 	return 0 unless $_[0]-> {can_draw};
 	my ( $self, $image, $x, $y, $xFrom, $yFrom, $xDestLen, $yDestLen, $xLen, $yLen, $rop) = @_;
+	$rop //= rop::Default;
+	$rop = $image->get_effective_rop($rop);
 	return 1 if $rop == rop::NoOper;
 
 	$image = $self->prepare_image($image, $xFrom, $yFrom, $xLen, $yLen);
@@ -1461,15 +1453,27 @@ Prima::PS::PDF -  PDF interface to Prima::Drawable
 
 =head1 SYNOPSIS
 
-	use Prima;
-	use Prima::PS::PDF;
+Recommended usage:
 
+	use Prima::PS::Printer;
+	my $x = Prima::PS::PDF::File-> new( file => 'out.pdf');
+
+or
+
+	my $x = Prima::PS::PDF::FileHandle-> new( handle => \&STDOUT);
+
+Low-level:
+
+	use Prima::PS::PDF;
 	my $x = Prima::PS::PDF-> create( onSpool => sub {
 		open F, ">> ./test.pdf";
 		binmode F;
 		print F $_[1];
 		close F;
 	});
+
+Printing:
+
 	die "error:$@" unless $x-> begin_doc;
 	$x-> font-> size( 30);
 	$x-> text_out( "hello!", 100, 100);
@@ -1478,9 +1482,9 @@ Prima::PS::PDF -  PDF interface to Prima::Drawable
 
 =head1 DESCRIPTION
 
-Realizes the Prima library interface to PDF v1.4.
-The module is designed to be compliant with Prima::Drawable interface.
-All properties' behavior is as same as Prima::Drawable's, except those
+Implements the Prima library interface to PDF v1.4.
+The module is designed to be compliant with the Prima::Drawable interface.
+All properties' behavior is as same as Prima::Drawable's except those
 described below.
 
 =head2 Inherited properties
@@ -1489,9 +1493,9 @@ described below.
 
 =item ::resolution
 
-Can be set while object is in normal stage - cannot be changed if document
-is opened. Applies to fillPattern realization and general pixel-to-point
-and vice versa calculations
+Can be set while the object is in the normal stage - cannot be changed if the
+document is opened. Applies to implementation of the fillPattern property and
+general pixel-to-point and vice versa calculations
 
 =back
 
@@ -1499,20 +1503,20 @@ and vice versa calculations
 
 =over
 
-=item ::grayscale
+=item grayscale
 
 could be 0 or 1
 
-=item ::pageSize
+=item pageSize
 
-physical page dimension, in points
+The physical page dimension, in points
 
-=item ::pageMargins
+=item pageMargins
 
-non-printable page area, an array of 4 integers:
-left, bottom, right and top margins in points.
+Non-printable page area, an array of 4 integers:
+left, bottom, right, and top margins in points.
 
-=item ::reversed
+=item reversed
 
 if 1, a 90 degrees rotated document layout is assumed
 
@@ -1528,15 +1532,16 @@ Helpers for translation from pixel to points and vice versa.
 
 =item spool
 
-Prima::PS::Drawable is not responsible for output of
-generated document, it just calls ::spool when document
-is closed through ::end_doc. By default just skips data.
-Prima::PS::Printer handles spooling logic.
+Prima::PS::PDF is not responsible for the output of the generated
+document, it only calls the C<::spool> method when the document is closed
+through an C<::end_doc> call. By default discards the data.  The
+C<Prima::PS::Printer> class handles the spooling logic.
 
 =item fonts
 
-Returns Prima::Application::fonts, however with C<iso10646-1> encoding only.
-That effectively allows only unicode output.
+Calls Prima::Application::fonts and returns its output filtered so that only
+the fonts that support the C<iso10646-1> encoding are present. This effectively
+allows only unicode output.
 
 =back
 
