@@ -1,5 +1,8 @@
 package Prima::VB::Config;
 
+use strict;
+use warnings;
+
 sub pages
 {
 	return qw(General Additional Sliders Abstract);
@@ -15,6 +18,13 @@ sub classes
 		RTModule => 'Prima::Sliders',
 		module   => 'Prima::VB::CoreClasses',
 	},
+	'Prima::ProgressBar' => {
+		icon     => 'VB::classes.gif:34',
+		page     => 'Sliders',
+		RTModule => 'Prima::Sliders',
+		class    => 'Prima::VB::ProgressBar',
+		module   => 'Prima::VB::CoreClasses',
+	},
 	'Prima::StringOutline' => {
 		icon     => 'VB::classes.gif:17',
 		page     => 'General',
@@ -50,11 +60,11 @@ sub classes
 		RTModule => 'Prima::InputLine',
 		module   => 'Prima::VB::CoreClasses',
 	},
-	'Prima::ScrollWidget' => {
+	'Prima::Widget::ScrollWidget' => {
 		icon     => 'VB::classes.gif:21',
 		page     => 'Abstract',
 		class    => 'Prima::VB::ScrollWidget',
-		RTModule => 'Prima::ScrollWidget',
+		RTModule => 'Prima::Widget::ScrollWidget',
 		module   => 'Prima::VB::CoreClasses',
 	},
 	'Prima::Widget' => {
@@ -89,7 +99,7 @@ sub classes
 		icon     => 'VB::classes.gif:6',
 		page     => 'Additional',
 		class    => 'Prima::VB::DirectoryListBox',
-		RTModule => 'Prima::FileDialog',
+		RTModule => 'Prima::Dialog::FileDialog',
 		module   => 'Prima::VB::CoreClasses',
 	},
 	'Prima::ComboBox' => {
@@ -152,7 +162,7 @@ sub classes
 		icon     => 'VB::classes.gif:1',
 		page     => 'Additional',
 		class    => 'Prima::VB::ColorComboBox',
-		RTModule => 'Prima::ColorDialog',
+		RTModule => 'Prima::Dialog::ColorDialog',
 		module   => 'Prima::VB::CoreClasses',
 	},
 	'Prima::Slider' => {
@@ -169,11 +179,11 @@ sub classes
 		RTModule => 'Prima::Sliders',
 		module   => 'Prima::VB::CoreClasses',
 	},
-	'Prima::Header' => {
+	'Prima::Widget::Header' => {
 		icon     => 'VB::classes.gif:30',
 		page     => 'Sliders',
 		class    => 'Prima::VB::Header',
-		RTModule => 'Prima::Header',
+		RTModule => 'Prima::Widget::Header',
 		module   => 'Prima::VB::CoreClasses',
 	},
 	'Prima::DirectoryOutline' => {
@@ -229,7 +239,7 @@ sub classes
 		icon     => 'VB::classes.gif:5',
 		page     => 'Additional',
 		class    => 'Prima::VB::DriveComboBox',
-		RTModule => 'Prima::FileDialog',
+		RTModule => 'Prima::Dialog::FileDialog',
 		module   => 'Prima::VB::CoreClasses',
 	},
 	'Prima::SpinButton' => {
@@ -253,6 +263,49 @@ sub classes
 		icon   => 'VB::classes.gif:33',
 		module   => 'Prima::VB::CoreClasses',
 	},
+	'Prima::DetailedOutline' => {
+		icon     => 'VB::classes.gif:35',
+		page     => 'Additional',
+		class    => 'Prima::VB::DetailedOutline',
+		RTModule => 'Prima::DetailedOutline',
+		module   => 'Prima::VB::CoreClasses',
+	},
+	'Prima::Widget::Date' => {
+		icon     => 'VB::classes.gif:36',
+		page     => 'Additional',
+		class    => 'Prima::VB::Date',
+		RTModule => 'Prima::Widget::Date',
+		module   => 'Prima::VB::CoreClasses',
+	},
+	'Prima::CheckList' => {
+		icon     => 'VB::classes.gif:37',
+		page     => 'Additional',
+		class    => 'Prima::VB::CheckList',
+		RTModule => 'Prima::ExtLists',
+		module   => 'Prima::VB::CoreClasses',
+	},
+	'Prima::KeySelector' => {
+		icon     => 'VB::classes.gif:38',
+		page     => 'Additional',
+		class    => 'Prima::VB::KeySelector',
+		RTModule => 'Prima::KeySelector',
+		module   => 'Prima::VB::CoreClasses',
+	},
+	'Prima::FrameSet' => {
+		icon     => 'VB::classes.gif:39',
+		page     => 'Sliders',
+		class    => 'Prima::VB::FrameSet',
+		RTModule => 'Prima::FrameSet',
+		module   => 'Prima::VB::CoreClasses',
+	},
+	'Prima::Widget::Time' => {
+		icon     => 'VB::classes.gif:40',
+		page     => 'Additional',
+		class    => 'Prima::VB::Time',
+		RTModule => 'Prima::Widget::Time',
+		module   => 'Prima::VB::CoreClasses',
+	},
+
 	);
 }
 
