@@ -44,22 +44,37 @@ rgn_rect( HV * profile, Bool is_box, unsigned int * n_boxes )
 }
 
 static PRegionRec
-rgn_polygon( HV * profile )
+rgn_polygon( SV * polygon, int fill_mode, double * matrix )
 {
-	dPROFILE;
 	Bool do_free;
-	int count, fill_mode;
+	int count;
 	PRegionRec rgn;
 	Point *points;
 
-	if (( points = (Point*) prima_read_array(
-		pget_sv(polygon), "Region::polygon", 'i',
-		2, 2, -1,
-		&count, &do_free)
-	) == NULL)
-		return NULL;
+	if ( matrix ) {
+		NPoint *npoints;
+		if (( npoints = (NPoint*) prima_read_array(
+			polygon, "Region::polygon", 'd',
+			2, 2, -1,
+			&count, &do_free)
+		) == NULL)
+			return NULL;
 
-	fill_mode = pexist(fillMode) ? pget_i(fillMode) : (fmOverlay | fmWinding);
+		if ( (points = malloc( count * sizeof(Point))) == NULL ) {
+			if ( do_free ) free( npoints );
+			return NULL;
+		}
+		prima_matrix_apply2_to_int( matrix, npoints, points, count );
+		if ( do_free ) free( npoints );
+		do_free = true;
+	} else {
+		if (( points = (Point*) prima_read_array(
+			polygon, "Region::polygon", 'i',
+			2, 2, -1,
+			&count, &do_free)
+		) == NULL)
+			return NULL;
+	}
 
 	rgn = img_region_polygon( points, count, fill_mode );
 	if ( do_free ) free( points );
@@ -67,6 +82,41 @@ rgn_polygon( HV * profile )
 	return rgn;
 }
 
+static PRegionRec
+rgn_polygons( SV * polygons, int fill_mode, double * matrix )
+{
+	AV *av;
+	int i, len;
+	PRegionRec ret = NULL;
+
+	if ( !polygons || !SvOK(polygons) || !SvROK(polygons) || SvTYPE(SvRV(polygons)) != SVt_PVAV)
+		return NULL;
+
+	av = (AV*) SvRV(polygons);
+	len = av_len(av);
+	for ( i = 0; i <= len; i ++ ) {
+		SV ** psv;
+		PRegionRec rgn;
+
+		if (( psv = av_fetch( av, i, 0 )) == NULL )
+			continue;
+
+		if (( rgn = rgn_polygon(*psv, fill_mode, matrix)) != NULL) {
+			if ( ret != NULL ) {
+				PRegionRec rgn2;
+				if (( rgn2 = img_region_combine( ret, rgn, rgnopUnion )) != NULL) {
+					free(ret);
+					ret = rgn2;
+				}
+				free(rgn);
+			} else
+				ret = rgn;
+		}
+	}
+
+	return ret;
+}
+
 static PRegionRec
 rgn_image( HV * profile )
 {
@@ -96,23 +146,43 @@ rgn_image( HV * profile )
 void
 Region_init( Handle self, HV * profile)
 {
-	Bool ok;
+	dPROFILE;
+	Bool ok, apply_matrix = true;
 	RegionRec r, *pr = &r;
+	double *matrix = NULL;
 
 	inherited-> init( self, profile);
 
-	r.flags = 0;
+	if ( pexist(matrix))
+		matrix = (double*) prima_read_array(
+			pget_sv(matrix),
+			"Region.create.matrix", 'd', 1, 6, 6, NULL, NULL
+		);
+
+	r.flags   = 0;
+	r.n_boxes = 0;
+	r.boxes   = NULL;
+
 	if ( pexist(rect)) {
 		r.boxes = rgn_rect(profile, 0, &r.n_boxes);
 	} else if (pexist(box)) {
 		r.boxes = rgn_rect(profile, 1, &r.n_boxes);
 	} else if (pexist(polygon)) {
-		pr = rgn_polygon(profile);
+		int fill_mode = pexist(fillMode) ? pget_i(fillMode) : (fmOverlay|fmWinding);
+		pr = rgn_polygon( pget_sv(polygon), fill_mode, matrix);
+		apply_matrix = false;
+	} else if (pexist(polygons)) {
+		int fill_mode = pexist(fillMode) ? pget_i(fillMode) : (fmOverlay|fmWinding);
+		pr = rgn_polygons( pget_sv(polygons), fill_mode, matrix);
+		apply_matrix = false;
 	} else if (pexist(image)) {
 		pr = rgn_image(profile);
-	} else {
-		r.n_boxes = 0;
-		r.boxes   = NULL;
+	}
+
+	if ( matrix ) {
+		if ( apply_matrix )
+			prima_matrix_apply2_int_to_int( matrix, (Point*) pr->boxes, (Point*) pr->boxes, pr->n_boxes * 2);
+		free(matrix);
 	}
 
 	ok = apc_region_create(self, pr);
