@@ -6,22 +6,21 @@ examples/dock.pl - Docking widgets
 
 =head1 FEATURES
 
-This is the demonstration of Prima::Dock and Prima::DockManager
-modules. The window created is docking client, and it's able
-to accept toolbars and panels, and toolbars in turn accept buttons.
-buttons are very samplish; there are two panels, Edit and Banner,
-that are docked in different ways.
-Note the following unevident features:
+A demo of the Prima::Dock and Prima::DockManager modules. The window is a
+docking client that is able to accept toolbars and panels as docked widdgets,
+and the toolbars in turn are able to accept buttons. Buttons are very
+primitive; there are two panels, Edit and Banner, that are docked using
+different mechanisms.  Note the following unevident features:
 
 =over 4
 
-=item popup on the border of the window ( and the Customize command there)
+=item popup on the border of the window (and the Customize command there)
 
 =item dragging of buttons on the window and the extreior
 
-=item dragging panels and toolbar to the exterior
+=item dragging panels and toolbar outside the window
 
-=item storing of the geometry in the ~/.demo_dock file
+=item storing and loading of the geometry in the ~/.demo_dock file
 
 =back
 
@@ -155,10 +154,9 @@ sub open_dockmanaging
 	my $self = $_[0];
 	my $i = $self-> instance;
 	return if $i-> interactiveDrag;
-	my $wpanel = Prima::Window-> create(
+	my $wpanel = Prima::Window-> new(
 		name => 'Customize tools',
-		size => [ 400, 100],
-		designScale => [ 7, 16 ],
+		size => [ 400, 200],
 		onClose => sub {
 			$self-> {toolBarPopup}-> customize-> enabled(1);
 			$i-> interactiveDrag(0);
@@ -347,7 +345,7 @@ sub on_paint
 package X;
 
 # createing the docking instance with predefined command state
-my $i = Prima::DockManager-> create(
+my $i = Prima::DockManager-> new(
 	commands  => {
 		'Edit::OK' => 0,
 		'Edit::Cancel' => 0,
@@ -400,7 +398,7 @@ my $resFile = Prima::Utils::path('demo_dock');
 
 # after all that, creating window ( the window itself is of small importance...)
 
-my $ww = Prima::Dock::BasicWindow -> create(
+my $ww = Prima::Dock::BasicWindow -> new(
 	instance => $i,
 	onClose => sub {
 		if ( open F, "> $resFile") {
@@ -442,6 +440,6 @@ $i-> predefined_toolbars( {
 
 #$ww-> open_dockmanaging; # uncomment this for Customize window popup immediately
 
-run Prima;
+Prima->run;
 
 1;
