@@ -1,13 +1,13 @@
-=pod 
+=pod
 
 =head1 NAME
 
-examples/matrix.pl - A matrix screen-saver
+examples/matrix.pl - matrix screen-saver
 
 =head1 FEATURES
 
-Tests the paletted DeviceBitmap implementation
-and large font output performance.
+Tests implementation of the paletted DeviceBitmap
+and performance of large fonts
 
 =cut
 
@@ -136,7 +136,7 @@ sub resetfs
 }
 
 
-my $w = Prima::MainWindow-> create(
+my $w = Prima::MainWindow-> new(
 	palette => [@spal],
 	font => { name => 'Courier New', size => $basicfsize, },
 	backColor => 0x002000,
@@ -191,7 +191,7 @@ my $w = Prima::MainWindow-> create(
 		if ( $showBitmaps) {
 			for ( $ymans = 0; $ymans < $vlines; $ymans++) {
 				my $y = $sz[1] - $vlbminst[ $ymans] * $fh;
-				$c-> put_image( $sz[0] - $vlbmxcol[ $ymans], $y, 
+				$c-> put_image( $sz[0] - $vlbmxcol[ $ymans], $y,
 					$dbms[ $vlbms[ $ymans]]);
 				if ( ++$vlbminst[ $ymans] >= $ymaxstep) {
 					$vlbminst[ $ymans] = 1;
@@ -258,7 +258,7 @@ my $w = Prima::MainWindow-> create(
 				ecolor( $c, $cc, cl::Yellow, 0.5) if $vlsped[ $ymans] > 1;
 				$y = $sz[1] - ($vlinst[ $ymans] - $maxln) * $fh;
 				for ( $i = 0; $i < $maxln; $i++) {
-					$c-> text_out( substr( $smp, $i, 1), 
+					$c-> text_out( substr( $smp, $i, 1),
 						$vlxcol[ $ymans] / $textToBMRatio, $y);
 					$y -= $fh;
 				}
@@ -282,4 +282,4 @@ $w-> insert( Timer =>
 })-> start;
 
 
-run Prima;
+Prima->run;
