@@ -2,12 +2,11 @@
 
 =head1 NAME
 
-examples/outline.pl - Prima outline widget
+examples/outline.pl - outline widget
 
 =head1 FEATURES
 
-Demonstrates the basic Prima toolkit usage
-and L<Prima::Outlines> class.
+Demonstrates the L<Prima::Outlines> widget
 
 =cut
 
@@ -15,7 +14,7 @@ use strict;
 use warnings;
 use Prima qw(Application Outlines);
 
-my $w = Prima::MainWindow-> create(
+my $w = Prima::MainWindow-> new(
 	size => [ 200, 200],
 	designScale => [7, 16],
 );
@@ -47,7 +46,13 @@ my $o = $w-> insert(
 		['Toogle multi select' => sub {
 			$_[0]-> multiSelect( !$_[0]-> multiSelect);
 		}],
+		[],
+		[Style => [
+			[ '(plusminus' => 'Plus-minus' => sub { $_[0]->style('plusminus') } ],
+			[ '*triangle)' => 'Triangle' => sub { $_[0]->style('triangle') } ],
+		]],
 	],
+	style => 'triangle',
 	multiSelect => 0,
 	extendedSelect => 1,
 	path => '.',
@@ -81,4 +86,4 @@ my ( $i, $l) = $o-> get_item( 1);
 #$o-> expand_all;
 #$o-> path('e:\Prima');
 
-run Prima;
+Prima->run;
