@@ -1,13 +1,13 @@
-=pod 
+=pod
 
 =head1 NAME
 
-examples/socket.pl - Downloads a html file from a given URL and strips it to text.
+examples/socket.pl - downloads a html file from a given URL
 
 =head1 FEATURES
 
 Tests the functionality of Prima::File and its
-events - onRead, onWrite and onException. ( The latter is not exemplified )
+events, onRead and onWrite.
 
 Note that the toolkit interface is alive during the download.
 
@@ -19,7 +19,7 @@ use Prima qw(InputLine Edit Application);
 use Socket;
 
 
-my $w = Prima::MainWindow-> create(
+my $w = Prima::MainWindow-> new(
 	size => [ $::application-> width * 0.8, $::application-> height * 0.8],
 	text => 'Socket demo',
 );
@@ -51,7 +51,7 @@ sub parse
 }
 
 my $data;
-my $watcher = Prima::File-> create(
+my $watcher = Prima::File-> new(
 	onRead => sub {
 		my ($me,$fh) = @_;
 		my ($d,$l);
@@ -76,7 +76,7 @@ my $watcher = Prima::File-> create(
 		syswrite $fh, $r, length($r);
 		$me-> mask( fe::Read); # want no write notifications
 	},
-);    
+);
 
 my $il = $w-> insert( InputLine =>
 	text => '',
@@ -87,7 +87,7 @@ my $il = $w-> insert( InputLine =>
 	return unless $key == kb::Enter;
 	$me-> clear_event;
 	my $t = $me-> text;
-	
+
 	unless ($t =~ m/^(?:http:\/\/)?([^\/]*)((?:\/.*$)|$)/) {
 		$e-> text( "Invalid URL");
 		return;
@@ -98,12 +98,17 @@ my $il = $w-> insert( InputLine =>
 	$remote_file = '/' unless length $remote_file;
 	$watcher-> {remote_host} = $remote;
 	$watcher-> {remote_file} = $remote_file;
-	
+	$watcher-> mask( 0);
+
 	my $port = 80;
 	my $proto = getprotobyname('tcp');
 	my $iaddr;
 	my $tx = $w-> text;
 	$w-> text( "Resolving $remote...");
+	if ( $remote =~ m/^(.*):(\d+)$/) {
+		$port = $2;
+		$remote = $1;
+	}
 	unless ( $iaddr = inet_aton($remote)) {
 		$w-> text( $tx);
 		$e-> text( "Cannot resolve $remote");
@@ -124,11 +129,11 @@ my $il = $w-> insert( InputLine =>
 	$w-> text( $tx);
 	$e-> text( $data = '');
 	$watcher-> file( *SOCK);
-	$watcher-> mask( fe::Read | fe::Write);
+	$watcher-> mask( fe::Write);
 }
 );
 
 $il-> text("Enter URL:");
 $il-> select_all;
 
-run Prima;
+Prima->run;
