@@ -5,18 +5,9 @@
 extern "C" {
 #endif
 
-#define FILL_PATTERN_SIZE 8
+#define FILL_PATTERN_SIZE sizeof(FillPattern)
 #define BLT_BUFSIZE ((MAX_SIZEOF_PIXEL * FILL_PATTERN_SIZE * FILL_PATTERN_SIZE) * 2)
 
-static void
-multiply( Byte * src, Byte * alpha, int alpha_step, Byte * dst, int bytes)
-{
-	while (bytes--) {
-		*(dst++) = *(src++) * *alpha / 255.0 + .5;
-		alpha += alpha_step;
-	}
-}
-
 typedef struct {
 	int bpp, als, dls, step, pat_x_offset;
 	Byte * dst, *dstMask, *pattern_buf, *adbuf;
@@ -75,7 +66,7 @@ img_bar_alpha_single_opaque( int x, int y, int w, int h, ImgBarAlphaCallbackRec
 
 		if ( a ) {
 			if ( ptr->dst_alpha_mul < 255 )
-				multiply( a, &ptr->dst_alpha_mul, 0, a, w);
+				img_multiply_alpha( a, &ptr->dst_alpha_mul, 0, a, w);
 			ptr->blend2(
 				&ptr->src_alpha, 0,
 				&ptr->src_alpha, 0,
@@ -122,7 +113,7 @@ img_bar_alpha_single_transparent( int x, int y, int w, int h, ImgBarAlphaCallbac
 				blt_bytes);
 			if ( a ) {
 				if ( ptr->dst_alpha_mul < 255 )
-					multiply( a, &ptr->dst_alpha_mul, 0, a, w);
+					img_multiply_alpha( a, &ptr->dst_alpha_mul, 0, a, w);
 				ptr->blend2(
 					&ptr->src_alpha, 0,
 					&ptr->src_alpha, 0,
@@ -373,7 +364,7 @@ img_bar_single( int x, int y, int w, int h, ImgBarCallbackRec * ptr)
 				pat_ptr = ptr->buf;
 			break;
 		default:
-			pat_ptr = ptr->buf + dx * ptr->bpp / 8;
+			pat_ptr = ptr->buf + dx * ptr->count;
 			if ( dx > 0 || blt_step + FILL_PATTERN_SIZE * ptr->count > BLT_BUFSIZE )
 				blt_step -= FILL_PATTERN_SIZE * ptr->count;
 		}
@@ -764,7 +755,7 @@ alpha_tiler( int x, int y, int w, int h, TileCallbackRec* ptr)
 		if ( !ptr->use_dst_alpha ) {
 			img_fill_alpha_buf( ptr->adbuf, a, w, bpp);
 			if ( ptr-> dst_alpha_mul < 255 )
-				multiply( ptr->adbuf, &ptr->dst_alpha_mul, 0, ptr->adbuf, bytes);
+				img_multiply_alpha( ptr->adbuf, &ptr->dst_alpha_mul, 0, ptr->adbuf, bytes);
 		}
 
 		/*
@@ -787,7 +778,7 @@ alpha_tiler( int x, int y, int w, int h, TileCallbackRec* ptr)
 
 		if (a != NULL) {
 			if ( ptr->dst_alpha_mul < 255 )
-				multiply( a, &ptr->dst_alpha_mul, 0, a, w);
+				img_multiply_alpha( a, &ptr->dst_alpha_mul, 0, a, w);
 			ptr->blend2(
 				ptr->use_src_alpha ? &ptr->src_alpha_mul : m,
 				ptr->use_src_alpha ? 0 : 1,
@@ -1176,18 +1167,20 @@ img_bar_pattern( Handle dest, int x, int y, int w, int h, PImgPaintContext ctx)
 
 	/* render a 8x8xPIXEL matrix with pattern, then horizontally
 	replicate it over blt_buffer as much as possible, to streamline
-	byte operations */
+	byte operations. Make sure that blt_step can adapt at least two
+	set of replcated patterns, as the later code may shift the point of
+	reference one pattern to the left */
 	switch ( i->type & imBPP) {
 	case imbpp1:
-		blt_bytes = (( x + w - 1) >> 3) - (x >> 3) + 1;
+		blt_bytes = (( x + w + 8 - 1) >> 3) - (x >> 3) + 1;
 		if ( blt_bytes < FILL_PATTERN_SIZE ) blt_bytes = FILL_PATTERN_SIZE;
 		break;
 	case imbpp4:
-		blt_bytes = (( x + w - 1) >> 1) - (x >> 1) + 1;
+		blt_bytes = (( x + w + 8 - 1) >> 1) - (x >> 1) + 1;
 		if ( blt_bytes < FILL_PATTERN_SIZE / 2 ) blt_bytes = FILL_PATTERN_SIZE / 2;
 		break;
 	default:
-		blt_bytes = w * pixSize;
+		blt_bytes = (w + 8) * pixSize;
 		if ( blt_bytes < FILL_PATTERN_SIZE * pixSize ) blt_bytes = FILL_PATTERN_SIZE * pixSize;
 	}
 	blt_bytes *= FILL_PATTERN_SIZE;
