@@ -747,11 +747,13 @@ img_polyline2patterns( NPoint * points, int n_points, Byte *lj_hints, double lin
 
 	/* merge the tail with the head if they collide */
 	if ( closed && pattern[0] > 1.0 && strokelen > 1.0 && curr != dst ) {
-		NPolyPolyline *p = dst;
+		NPolyPolyline *p = dst, *old_curr = curr;
 		dst = dst->next;
 		if ( p->n_points > curr-> size - curr-> n_points) {
 			if ( !( curr = nppl_alloc(curr, lj_hints != NULL, curr->size + p-> n_points)))
 				goto EXIT;
+			if ( old_curr == dst )
+				dst = curr;
 		}
 		memcpy( curr-> points + curr->n_points, p-> points, p->n_points * sizeof(NPoint));
 		if ( curr-> lj_hints)
