@@ -502,6 +502,7 @@ match_font( PFontKey fk, PFont font, Matrix matrix)
 	default:
 		return NULL;
 	}
+	XCHECKPOINT;
 
 	if ( !cf2 ) {
 		free(cf);
@@ -824,10 +825,13 @@ apc_font_get_glyph_bitmap( Handle self, uint16_t index, unsigned int flags, PPoi
 }
 
 Bool
-apc_font_is_colored( Handle self)
+apc_font_has_color( Handle self)
 {
 	DEFXX;
 	PCachedFont f = XX->font;
+
+	if ( !f )
+		return false;
 	if (
 		f->has_colors < 0 ||
 		( XT_IS_DBM(XX) && XT_IS_BITMAP(XX)) ||
@@ -839,7 +843,7 @@ apc_font_is_colored( Handle self)
 
 #ifdef USE_FONTQUERY
 	if ( is_opt(optInFontQuery) ) {
-		Bool ok = prima_ft_is_font_colored(X(self)->font->ft_face);
+		Bool ok = prima_ft_font_has_color(X(self)->font->ft_face);
 		f->has_colors = ok ? 1 : -1;
 		return ok;
 	}
@@ -847,7 +851,7 @@ apc_font_is_colored( Handle self)
 
 #ifdef USE_XFT
 	if ( f->xft ) {
-		Bool ok = prima_xft_is_font_colored(self);
+		Bool ok = prima_xft_font_has_color(self);
 		f->has_colors = ok ? 1 : -1;
 		return ok;
 	}
