@@ -573,6 +573,7 @@ prima_fc_fonts( PFont array, const char *facename, const char * encoding, int *r
 	PFont newarray, f;
 	PHash names = NULL;
 	CharSetInfo * csi = NULL;
+	size_t newsize;
 	int i;
 
 	if ( encoding) {
@@ -598,8 +599,8 @@ prima_fc_fonts( PFont array, const char *facename, const char * encoding, int *r
 	}
 
 	/* make dynamic */
-	i = sizeof(Font) * (*retCount + s-> nfont * ALL_CHARSETS);
-	newarray = array ? realloc(array,i) : malloc(i);
+	newsize = sizeof(Font) * (*retCount + s-> nfont * ALL_CHARSETS);
+	newarray = array ? realloc(array,newsize) : malloc(newsize);
 	if ( !newarray) {
 		FcFontSetDestroy(s);
 		return array;
@@ -785,10 +786,11 @@ prima_fc_get_font_languages( FcPattern *pat)
 		int len = strlen((char*)s);
 		if ( p - ret + len + 1 + 1 > size ) {
 			char * p2;
+			ptrdiff_t diff = p - ret;
 			size *= 2;
 			if ( !( p2 = realloc(ret, size)))
 				goto FAIL;
-			p   = p2 + (p - ret);
+			p   = p2 + diff;
 			ret = p2;
 		}
 		strcpy( p, (char*) s );
