@@ -57,8 +57,8 @@ TO DO
 #endif
 
 #ifdef WITH_HARFBUZZ
-#include <harfbuzz/hb.h>
-#include <harfbuzz/hb-ft.h>
+#include <hb.h>
+#include <hb-ft.h>
 #endif
 
 #define MY_MATRIX (PDrawable(self)->current_state.matrix)
@@ -491,10 +491,6 @@ prima_xft_match( Font *font, Matrix matrix, Bool by_size, PCachedFont cf)
 			XFTdebug("set height: %d", font->height);
 		}
 		font-> maximalWidth = xf-> max_advance_width;
-		/* XXX FT_Face reports very strange values */
-		font->underlinePosition  = -xf-> descent;
-		font->underlineThickness = (font->height > 16) ? font->height / 16 : 1;
-
 		{
 			FT_Face f;
 			if ( ( f = XftLockFace(xf)) != NULL) {
@@ -1402,7 +1398,7 @@ prima_xft_get_glyph_outline( Handle self, unsigned int index, unsigned int flags
 	c = ((flags & (ggoUnicode|ggoGlyphIndex)) || index <= 128) ? index : XX-> fc_map8[index - 128];
 	ft_index = (flags & ggoGlyphIndex) ? c : XftCharIndex( DISP, XX->font->xft, c);
 
-	ret = prima_ft_get_glyph_outline( ft_face, ft_index, ft_flags, buffer);
+	ret = prima_ft_get_glyph_outline( ft_face, ft_index, ft_flags, flags & ggoARGB, buffer);
 
 	XftUnlockFace(XX->font->xft);
 
@@ -1429,14 +1425,14 @@ prima_xft_get_glyph_bitmap( Handle self, uint16_t index, unsigned int flags, PPo
 }
 
 Bool
-prima_xft_is_font_colored( Handle self )
+prima_xft_font_has_color( Handle self )
 {
 	DEFXX;
 	Bool ret;
 	FT_Face face;
 	if ( !( face = XftLockFace( XX->font->xft)))
 		return false;
-	ret = prima_ft_is_font_colored(face);
+	ret = prima_ft_font_has_color(face);
 	XftUnlockFace(XX->font->xft);
 	return ret;
 }
